/*
 * DBDocumentIndexParameters.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Ghostar, TCSDEVELOPER
 */

using System;

namespace TopCoder.Document.Index.Persistence.DB
{
    /// <summary>
    /// Extends the <see cref="DBPersistenceParameters"/> class to provide an extra DocumentID property
    /// as a supplement to the properties contained by its parent class.
    /// </summary>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class DBDocumentIndexParameters : DBPersistenceParameters
    {
        /// <summary>
        /// This member variable holds the document ID of a specific document for
        /// the DBDocumentIndexSource instance to work against in the database.
        /// This value is set in the constructor and is accessed through the DocumentID property.
        /// </summary>
        private readonly long documentId;

        /// <summary>
        /// Gets the document id.
        /// </summary>
        ///
        /// <remarks>
        /// This property is used by the <see cref="DBDocumentIndexSource"/> class
        /// to load a specific document's information from the database.
        /// </remarks>
        ///
        /// <value>
        /// The document id.
        /// </value>
        public long DocumentID
        {
            get
            {
                return documentId;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DBDocumentIndexParameters"/>.
        /// </summary>
        ///
        /// <param name="connectionName">
        /// Name of the connection to use.
        /// </param>
        /// <param name="documentId">
        /// The ID of the document.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="connectionName"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="connectionName"/> is an empty trimmed string
        /// or <paramref name="documentId"/> is negative.
        /// </exception>
        public DBDocumentIndexParameters(string connectionName, long documentId)
            : base(connectionName)
        {
            if (documentId < 0)
            {
                throw new ArgumentException("'documentId' cannot be negative", "documentId");
            }
            this.documentId = documentId;
        }

        /// <summary>
        /// Serves as a hash function for a particular type,
        /// suitable for use in hashing algorithms and data structures like a hash table.
        /// </summary>
        ///
        /// <returns>
        /// A hash code of this instance.
        /// </returns>
        public override int GetHashCode()
        {
            return (ConnectionName.GetHashCode() ^ DocumentID.GetHashCode());
        }

        /// <summary>
        /// Determines whether two <see cref="DBDocumentIndexParameters"/> instances are equal.
        /// </summary>
        ///
        /// <param name="obj">
        /// Object to compare to.
        /// </param>
        ///
        /// <returns>
        /// true if the specified object is equal to this instance; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            if (!(obj is DBDocumentIndexParameters))
            {
                return false;
            }

            DBDocumentIndexParameters p = (DBDocumentIndexParameters) obj;
            return ((ConnectionName == p.ConnectionName) && (DocumentID == p.DocumentID));
        }
    }
}
