/*
 * DBDocumentIndexSource.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Ghostar, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Data;
using System.Globalization;
using TopCoder.Data.ConnectionFactory;
using TopCoder.Document.Index.WordStreams;

namespace TopCoder.Document.Index.Persistence.DB
{
    /// <summary>
    /// Implements the <see cref="IDocumentIndexSource"/> which provides document index persistence
    /// in a database.
    /// </summary>
    ///
    /// <remarks>
    /// It works with three tables in the database: Document, Location, and Word.
    /// Note that all changes are in a transaction, and <see cref="Save"/> is used to commit it.
    /// This class is mutable.
    /// Note that if the document ID given in constructor or Load() doesn't exist yet in the database,
    /// it won't be possible to add words. Please use <see cref="DBCollectionIndexSource"/>
    /// to add the document collection into the database.
    /// </remarks>
    ///
    /// <example>
    /// Here's a sample configuration file for this class: <code><![CDATA[
    /// <?xml version="1.0" encoding="UTF-8"?>
    /// <!DOCTYPE ConfigManager SYSTEM "cm.dtd">
    ///
    /// <ConfigManager>
    ///     <!-- a sample valid configuration -->
    ///     <namespace name="TopCoder.Document.Index.Persistence.DB">
    ///         <!-- the db connection name (req) -->
    ///         <property name="DBConnectionName">
    ///             <value>InformixOdbc</value>
    ///         </property>
    ///         <property name="DocumentID">
    ///             <value>1</value>
    ///         </property>
    ///     </namespace>
    /// </ConfigManager>
    /// ]]></code>
    /// Here's a sample code: <code><![CDATA[
    /// IDocumentIndexSource source = null;
    /// try
    /// {
    ///     // create a new document collection first
    ///     ICollectionIndexSource dbColl = new DBCollectionIndexSource();
    ///     IParameters colParams = new DBPersistenceParameters(DbConnectionName);
    ///     dbColl.Load(colParams);
    ///
    ///     // save and re-load to get the document params
    ///     IParameters documentParams = new DBDocumentIndexParameters(DbConnectionName, 1);
    ///     IndexInfo info = new IndexInfo(typeof(DBDocumentIndexSource).Assembly.GetName().Name,
    ///         typeof(DBDocumentIndexSource).FullName, documentParams);
    ///     dbColl.Add("New document", info);
    ///     dbColl.Save();
    ///     dbColl.Load(colParams);
    ///     documentParams = dbColl["New document"].Parameters;
    ///
    ///     // load document words' indices (initially empty)
    ///     source = new DBDocumentIndexSource();
    ///     source.Load(documentParams);
    ///
    ///     // check CompareOptions and Culture
    ///     CompareOptions opt = source.CompareOptions;
    ///     CultureInfo ci = source.Culture;
    ///
    ///     // check IndexInfo
    ///     IndexInfo idxInfo = source.IndexInfo;
    ///
    ///     // enumerate the word indices
    ///     IDictionaryEnumerator enumerator = source.GetEnumerator();
    ///     int count = 0;
    ///     while (enumerator.MoveNext())
    ///     {
    ///         count++;
    ///     }
    ///
    ///     // add a new word
    ///     WordInfo word = new WordInfo("Cat", 102);
    ///     source.Add(word);
    ///
    ///     // add using indexer
    ///     IList pos = new ArrayList();
    ///     pos.Add(103);
    ///     pos.Add(105);
    ///     source["Dog"] = pos;
    ///
    ///     // save changes
    ///     source.Save();
    ///
    ///     // check word exists
    ///     bool exists = source.Contains("Cat");
    ///
    ///     // remove a word
    ///     source.Remove("Cat");
    ///
    ///     // use indexer
    ///     IList getPos = source["Dog"];
    ///     int count = getPos.Count;
    ///
    ///     // clear all words
    ///     source.Clear();
    ///
    ///     // save transaction
    ///     source.Save();
    /// }
    /// finally
    /// {
    ///     if (source != null)
    ///     {
    ///         // dispose db resources
    ///         ((IDisposable) source).Dispose();
    ///     }
    /// }
    /// ]]></code>
    /// </example>
    ///
    /// <threadsafety static="false" instance="false"/>
    public class DBDocumentIndexSource : IDocumentIndexSource, IComparer, IDisposable
    {
        /// <summary>
        /// The default namespace used to load configuration values from, if one is not provided.
        /// </summary>
        public static readonly string DefaultNamespace = "TopCoder.Document.Index.Persistence.DB";

        /// <summary>
        /// The config key for the name of the db connection in the Connection Factory component.
        /// </summary>
        private const string DBConnectionNameConfig = "DBConnectionName";

        /// <summary>
        /// The config key for the document id.
        /// </summary>
        private const string DocumentIDConfig = "DocumentID";

        /// <summary>
        /// SQL statement to select word_id from Word table.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlSelectWord = new SqlStatement(
            "SELECT Word_Id FROM Word WHERE Word = ?", "Word");

        /// <summary>
        /// SQL statement to insert a word into the Word table.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlInsertWord = new SqlStatement(
            "INSERT INTO Word (Word) VALUES (?)", "Word");

        /// <summary>
        /// SQL statement to select a location from the Location table.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlSelectLocation = new SqlStatement(
            "SELECT Location_Id FROM Location "
            + "WHERE Word_Id = ? AND Document_Id = ? AND Location = ?",
            "Word_Id", "Document_Id", "Location");

        /// <summary>
        /// SQL statement to insert a location into the Location table.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlInsertLocation = new SqlStatement(
            "INSERT INTO Location (Word_Id, Document_Id, Location) "
            + "VALUES (?, ?, ?)", "Word_Id", "Document_Id", "Location");

        /// <summary>
        /// SQL statement to delete all locations from the Location table
        /// for the specified word id and document id.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlDeleteLocationByWordIdAndDocId = new SqlStatement(
            "DELETE FROM Location "
            + "WHERE Word_Id = ? AND Document_Id = ?",
            "Word_Id", "Document_Id");

        /// <summary>
        /// SQL statement to delete all locations from the Location table
        /// for the specified document id.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlDeleteLocationByDocId = new SqlStatement(
            "DELETE FROM Location "
            + "WHERE Document_Id = ?",
            "Document_Id");

        /// <summary>
        /// SQL statement to select document's CompareOptions and Culture.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlSelectDocument = new SqlStatement(
            "SELECT Compare_Options, Culture FROM Document "
            + "WHERE Document_Id = ?",
            "Document_Id");

        /// <summary>
        /// SQL statement to select all words-location from the Word/Location table for a document id.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlSelectWordLocation = new SqlStatement(
            "SELECT Word, Location FROM Location INNER JOIN Word ON Location.Word_Id = Word.Word_Id "
            + "WHERE Document_Id = ?",
            "Document_Id");

        /// <summary>
        /// SQL statement to update document's CompareOptions and Culture.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlUpdateDocument = new SqlStatement(
            "UPDATE Document "
            + "SET Compare_Options = ?, Culture = ? "
            + "WHERE Document_Id = ?",
            "Compare_Options", "Culture", "Document_Id");

        /// <summary>
        /// This private member variables holds the culture to use when comparing two strings.
        /// This allows us to provide support for multiple languages and alphabets
        /// when searching through a given document.
        /// This member variable is initialized to a default value, and
        /// it can be changed through the Culture property.
        /// </summary>
        private CultureInfo culture = new CultureInfo(Helper.DefaultCulture);

        /// <summary>
        /// This member variable holds options, beyond the culture, for comparing strings
        /// in this particular class. This value is initialized to a default,
        /// and it can be accessed using the CompareOptions property.
        /// </summary>
        private CompareOptions compareOptions = Helper.DefaultCompareOptions;

        /// <summary>
        /// This variable holds the configuration information for this class,
        /// including the name of the connection loaded, as well as the document ID.
        /// This value is initialized to a new DBDocumentIndexParameters instance
        /// in the constructor, using values read from the namespace given to the constructor.
        /// It can also be changed via the Load method.
        /// </summary>
        private DBDocumentIndexParameters parameters;

        /// <summary>
        /// This connection is the actual connection that is used to talk to the database for updates.
        /// This value is set using the Connection Factory component in the constructors.
        /// The System.Data.Odbc classes should be used when executing commands against this connection,
        /// as we need to be able to connect to an Informix database, as specified in the RS.
        /// The connection will be closed as soon as the transaction is committed/rolled-back.
        /// </summary>
        private IDbConnection connection;

        /// <summary>
        /// All updates are added to this transaction, and they are committed in the Save() method.
        /// This is lazily initialized to a new IDbTransaction implementation using
        /// IDbConnection.BeginTransaction() method whenever needed.
        /// After transaction is committed/rolled-back, this will be initialized to null
        /// and connection will be closed to ensure no db resource is left hanging.
        /// </summary>
        private IDbTransaction transaction;

        /// <summary>
        /// This member variable holds the loaded words, loaded from the database,
        /// that this instance is working against. The keys in this dictionary are the words,
        /// and the values are ILists of long locations in the document where this Word can be found.
        /// This collection is loaded by the Load method and accessed by the indexers in this class.
        /// </summary>
        private IDictionary words;

        /// <summary>
        /// Tracks whether Dispose has been called.
        /// </summary>
        private bool isDisposed = false;

        /// <summary>
        /// Gets a newly created <see cref="IndexInfo"/> instance, created with the assembly
        /// and file name of this class, along with a <see cref="DBDocumentIndexParameters"/>
        /// instance that holds the connection name and document ID that the current instance
        /// is currently working against.
        /// </summary>
        ///
        /// <value>
        /// An <see cref="IndexInfo"/> instance, created with the assembly
        /// and file name of this class, along with a <see cref="DBDocumentIndexParameters"/>
        /// instance that holds the connection name and document ID that the current instance
        /// is currently working against.
        /// </value>
        public IndexInfo IndexInfo
        {
            get
            {
                // create a new IndexInfo with parameters
                return new IndexInfo(
                    this.GetType().Assembly.FullName,
                    this.GetType().FullName,
                    parameters);
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="CultureInfo"/> which is used for string comparisons.
        /// </summary>
        ///
        /// <value>
        /// The <see cref="CultureInfo"/> which is used for string comparisons.
        /// </value>
        ///
        /// <remarks>
        /// The default value is CultureInfo("en-us").
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// Setting to a null reference.
        /// </exception>
        /// <exception cref="InvalidOperationException">
        /// This document words are not empty.
        /// </exception>
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while updating the document's culture.
        /// </exception>
        public CultureInfo Culture
        {
            get
            {
                return culture;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("Culture", "'Culture' cannot be null");
                }
                if (words.Count != 0)
                {
                    throw new InvalidOperationException(
                        "Document words exist already, cannot change Culture");
                }
                DbUpdateCompareOptionsAndCulture(CompareOptions, value);
                culture = value;
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="CompareOptions"/>, which is used in conjunction
        /// with the <see cref="Culture"/> property for string comparison options.
        /// </summary>
        ///
        /// <value>
        /// The <see cref="CompareOptions"/>, which is used in conjunction
        /// with the <see cref="Culture"/> property for string comparison options.
        /// </value>
        ///
        /// <remarks>
        /// The default value is CompareOptions.None.
        /// </remarks>
        ///
        /// <exception cref="InvalidOperationException">
        /// This document words are not empty.
        /// </exception>
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while updating the document's compare options.
        /// </exception>
        public CompareOptions CompareOptions
        {
            get
            {
                return compareOptions;
            }
            set
            {
                if (words.Count != 0)
                {
                    throw new InvalidOperationException(
                        "Document words exist already, cannot change CompareOptions");
                }
                DbUpdateCompareOptionsAndCulture(value, Culture);
                compareOptions = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DBDocumentIndexSource"/> class.
        /// </summary>
        ///
        /// <remarks>
        /// This constructor creates a new <see cref="DBDocumentIndexSource"/> instance
        /// using the default namespace <see cref="DefaultNamespace"/> constant.
        /// The instance will initially be blank.
        /// The <see cref="Load"/> method should be used to load the actual data in
        /// for this class.
        /// </remarks>
        ///
        /// <exception cref="DBPersistenceConfigurationException">
        /// The default namespace doesn't contain the required configuration parameters.
        /// </exception>
        public DBDocumentIndexSource()
            : this(DefaultNamespace)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DBDocumentIndexSource"/> class
        /// based on configuration parameters read from the given namespace.
        /// </summary>
        ///
        /// <remarks>
        /// Note that <see cref="Load"/> method should be used to load the actual data in
        /// for this class.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// Namespace to load configuration from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="nameSpace"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is an empty string.
        /// </exception>
        /// <exception cref="DBPersistenceConfigurationException">
        /// The namespace doesn't contain the required configuration parameters.
        /// </exception>
        public DBDocumentIndexSource(string nameSpace)
        {
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace", "'nameSpace' cannot be null");
            }
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("'nameSpace' cannot be an empty trimmed string", "nameSpace");
            }

            try
            {
                // get connection name from configuration file (req)
                string connName = Helper.GetCMValue(nameSpace, DBConnectionNameConfig);

                // get document id from configuration file (req)
                string docID = Helper.GetCMValue(nameSpace, DocumentIDConfig);

                // initialize parameters and create the DB connection
                parameters = new DBDocumentIndexParameters(connName, long.Parse(docID));
                connection = ConnectionManager.Instance.CreateDbConnection(connName, new Hashtable());

                // initialize words dictionary
                words = new SortedList(this);
            }
            catch (DBPersistenceConfigurationException)
            {
                // re-throw
                throw;
            }
            catch (Exception e)
            {
                throw new DBPersistenceConfigurationException(string.Format(
                    "Error configuring from namespace '{0}'", nameSpace), e);
            }
        }

        /// <summary>
        /// Get or sets a <see cref="IList"/> of positions (long) for a given word.
        /// </summary>
        ///
        /// <param name="word">
        /// The word to return or set the locations for.
        /// </param>
        ///
        /// <value>
        /// An <see cref="IList"/> of all locations (long) for the given word in this document
        /// or null if the word doesn't exist.
        /// </value>
        ///
        /// <remarks>
        /// Note that if a word already exists for the specified name, it will be replaced.
        /// If the given IList contains duplicate position, it'll be merged.
        /// The word will be trimmed before stored.
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="word"/> is a null reference or setting to a null value.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="word"/> is an empty trimmed string or
        /// the IList contains non-long/integer type.
        /// </exception>
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while setting the document words.
        /// </exception>
        public IList this[string word]
        {
            get
            {
                if (word == null)
                {
                    throw new ArgumentNullException("word", "'word' cannot be null");
                }
                if (word.Trim().Length == 0)
                {
                    throw new ArgumentException("'word' cannot be an empty trimmed string", "word");
                }

                return (IList) words[word];
            }
            set
            {
                if (word == null)
                {
                    throw new ArgumentNullException("word", "'word' cannot be null");
                }
                string key = word.Trim();
                if (key.Length == 0)
                {
                    throw new ArgumentException("'word' cannot be an empty trimmed string", "word");
                }
                if (value == null)
                {
                    throw new ArgumentNullException("Position list cannot be null");
                }
                foreach (object o in value)
                {
                    // only long/integer is accepted
                    if (!Helper.IsLong(o))
                    {
                        throw new ArgumentException(
                            "Position list contains non-long/integer element");
                    }
                }

                // insert into database, but first remove all existing location for this word
                Remove(key);    // this will clear words IDictionary as well

                // add new word
                IList posList = new ArrayList();
                words[key] = posList;
                foreach (object o in value)
                {
                    if (DbInsertWordPosition(key, Convert.ToInt64(o)))
                    {
                        // not a duplicate position
                        posList.Add(o);
                    }
                }
            }
        }

        /// <summary>
        /// Adds the given <see cref="WordInfo"/> to the database.
        /// </summary>
        ///
        /// <param name="wordInfo">
        /// The information about the word to be added to the persistent store.
        /// </param>
        ///
        /// <remarks>
        /// This word will first be added into the Word table, if it doesn't exist.
        /// If it does exist in that table, the ID of the word is retrieved, then,
        /// the location of this word is added into the Location table,
        /// using the word ID, location given, and the document ID of this class.
        /// Duplicate position will be ignored.
        /// The word will be trimmed before stored.
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="wordInfo"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// The given word is an empty trimmed string.
        /// </exception>
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while inserting the word/location into the database.
        /// </exception>
        public void Add(WordInfo wordInfo)
        {
            if (wordInfo == null)
            {
                throw new ArgumentNullException("wordInfo", "'wordInfo' cannot be null");
            }

            // get the position list (AE is checked by get-indexer)
            string key = wordInfo.Word.Trim();
            IList posList = this[key];

            // insert to database first
            bool ok = DbInsertWordPosition(key, wordInfo.Position);

            // add to words IDictionary
            if (posList == null)
            {
                // create a new list and store it
                posList = new ArrayList();
                words[key] = posList;
            }

            // add the position to the list (if the position doesn't exist in db yet)
            if (ok)
            {
                posList.Add(wordInfo.Position);
            }
        }

        /// <summary>
        /// Removes the given word and its locations from the Location table.
        /// </summary>
        ///
        /// <remarks>
        /// Nothing will occur if the specified word doesn't exist.
        /// Since the word table reference could be used by multiple documents,
        /// only rows in the Location table that have the id of this document and
        /// the ID of the word from the Word table are removed.
        /// </remarks>
        ///
        /// <param name="word">
        /// The word to remove from the persistent store.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="word"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="word"/> is an empty trimmed string.
        /// </exception>
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while deleting the word locations from the database.
        /// </exception>
        public void Remove(string word)
        {
            if (word == null)
            {
                throw new ArgumentNullException("word", "'word' cannot be null");
            }
            if (word.Trim().Length == 0)
            {
                throw new ArgumentException("'word' cannot be an empty trimmed string", "indexName");
            }

            DbDeleteLocationByWordIdAndDocId(word);
            words.Remove(word);
        }

        /// <summary>
        /// Determines whether a word exists in the document.
        /// </summary>
        ///
        /// <param name="word">
        /// The word to look for.
        /// </param>
        ///
        /// <returns>
        /// true if the document contains the given word; otherwise, false.
        /// There has to be at least 1 position defined for the given word, otherwise
        /// false is always returned.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="word"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="word"/> is an empty trimmed string.
        /// </exception>
        public bool Contains(string word)
        {
            // getter check ANE/AE
            IList pos = this[word];

            return ((pos == null) ? false : (pos.Count > 0));
        }

        /// <summary>
        /// Clears all words from the document.
        /// </summary>
        ///
        /// <remarks>
        /// This will remove all rows in the Location table with this document id.
        /// </remarks>
        ///
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while deleting the locations from the database.
        /// </exception>
        public void Clear()
        {
            DbDeleteLocationByDocId();
            words.Clear();
        }

        /// <summary>
        /// Returns the enumerator for the words collection, which can be used by the user
        /// to enumerate through all words in this document.
        /// </summary>
        ///
        /// <returns>
        /// The enumerator for all words in the index.
        /// </returns>
        public IDictionaryEnumerator GetEnumerator()
        {
            return words.GetEnumerator();
        }

        /// <summary>
        /// Loads the document index from the database.
        /// </summary>
        ///
        /// <remarks>
        /// The parameter given should be a <see cref="DBDocumentIndexParameters"/> instance
        /// that contains the connection name and document id to load for.
        /// </remarks>
        ///
        /// <param name="parameters">
        /// The <see cref="SimpleIndexParameters"/> containing the xml file path to load.
        /// </param>
        ///
        /// <remarks>
        /// This method loads the words with all values from the Location and Word tables
        /// that match the current documentId of this class.
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="parameters"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="parameters"/> is not a <see cref="DBDocumentIndexParameters"/> instance.
        /// contains invalid FileName parameters
        /// (malformed, points to a directory, doesn't exists).
        /// </exception>
        /// <exception cref="LoadIndexDataException">
        /// Any errors occur while loading the document words.
        /// </exception>
        public void Load(IParameters parameters)
        {
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters", "'parameters' cannot be null");
            }
            if (!(parameters is DBDocumentIndexParameters))
            {
                throw new ArgumentException("'parameters' must be of DBDocumentIndexParameters type", "parameters");
            }

            // create a new db connection from the parameters
            try
            {
                // but first rollback any pending transaction and close previous connection
                RollsbackTransaction();

                string connName = ((DBDocumentIndexParameters) parameters).ConnectionName;
                connection = ConnectionManager.Instance.CreateDbConnection(connName, new Hashtable());

                // reinitialize parameters
                this.parameters = (DBDocumentIndexParameters) parameters;
            }
            catch (Exception e)
            {
                throw new DBPersistenceConfigurationException(
                    "Error creating database connection", e);
            }

            // loads compare options and culture settings
            // then load all document words and populates indices IDictionary
            DbLoad();
        }

        /// <summary>
        /// Commits all changes to the database.
        /// </summary>
        ///
        /// <exception cref="SaveIndexDataException">
        /// An error occurs while committing the transaction.
        /// </exception>
        public void Save()
        {
            try
            {
                if (transaction != null)
                {
                    // commit, reset transaction to null, and close connection
                    transaction.Commit();
                    transaction = null;
                    connection.Close();
                }
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, "Error committing transaction");
            }
        }

        /// <summary>
        /// Compare two words using the culture info contained in this class.
        /// </summary>
        ///
        /// <remarks>
        /// This method is used by the sort the word indices in this document.
        /// The <see cref="CompareOptions"/> is also taken into account.
        /// </remarks>
        /// <param name="x">
        /// The first object to compare.
        /// </param>
        /// <param name="y">
        /// The second object to compare.
        /// </param>
        ///
        /// <returns>
        /// Result of the comparison.
        /// </returns>
        ///
        /// <exception cref="ArgumentException">
        /// Either parameter is not a string.
        /// </exception>
        public int Compare(object x, object y)
        {
            if (!(x is string))
            {
                throw new ArgumentException("'x' must be of string type", "x");
            }
            if (!(y is string))
            {
                throw new ArgumentException("'y' must be of string type", "y");
            }
            // compare using the Culture and CompareOptions
            return Culture.CompareInfo.Compare((string) x, (string) y, CompareOptions);
        }

        /// <summary>
        /// Disposes the various db resources.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            // tell GC not to run finalizer
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Finalizers which is called by the garbage collector.
        /// </summary>
        ~DBDocumentIndexSource()
        {
            // Do not re-create Dispose clean-up code here.
            // Calling Dispose(false) is optimal in terms of
            // readability and maintainability.
            Dispose(false);
        }

        /// <summary>
        /// Implements the IDisposable pattern as specified in the MSDN documentation.
        /// </summary>
        ///
        /// <param name="disposing">
        /// true if this is called by user (managed resource can be referenced),
        /// false if this is called by garbage collector (don't touch managed resource).
        /// </param>
        private void Dispose(bool disposing)
        {
            if (!isDisposed)
            {
                // can only reference managed resource if user invokes the dispose
                if (disposing)
                {
                    // rollback transaction and closes connection
                    RollsbackTransaction();

                    // dispose all SqlStatement
                    sqlSelectWord.Dispose();
                    sqlInsertWord.Dispose();
                    sqlSelectLocation.Dispose();
                    sqlInsertLocation.Dispose();
                    sqlDeleteLocationByWordIdAndDocId.Dispose();
                    sqlDeleteLocationByDocId.Dispose();
                    sqlSelectWordLocation.Dispose();
                    sqlSelectDocument.Dispose();
                    sqlUpdateDocument.Dispose();
                }
            }
            isDisposed = true;
        }

        /// <summary>
        /// Inserts into the Word/Location table.
        /// </summary>
        ///
        /// <param name="word">
        /// The word to insert.
        /// </param>
        /// <param name="pos">
        /// The location to insert.
        /// </param>
        ///
        /// <returns>
        /// true if the location is successfully added; false if the location already exists.
        /// </returns>
        ///
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while inserting the word/location.
        /// </exception>
        private bool DbInsertWordPosition(string word, long pos)
        {
            try
            {
                InitTransaction();

                // get word_id, insert it if it doesn't exist yet
                // SELECT Word_Id FROM Word WHERE Word = ?
                object wordId = sqlSelectWord.ExecuteScalar(connection, transaction, word);
                if (wordId == null)
                {
                    // INSERT INTO Word (Word) VALUES (?)", "Word") then retrieve the word_id
                    sqlInsertWord.ExecuteNonQuery(connection, transaction, word);
                    wordId = sqlSelectWord.ExecuteScalar(connection, transaction, word);
                }

                // insert position into Location table if it doesn't exist yet
                // SELECT Location_Id FROM Location
                // WHERE Word_Id = ? AND Document_Id = ? AND Location = ?
                object locationId = sqlSelectLocation.ExecuteScalar(connection, transaction,
                    wordId, parameters.DocumentID, pos);
                if (locationId == null)
                {
                    // INSERT INTO Location (Word_Id, Document_Id, Location) VALUES (?, ?, ?)
                    sqlInsertLocation.ExecuteNonQuery(connection, transaction,
                        wordId, parameters.DocumentID, pos);
                }

                return (locationId == null);
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, string.Format(
                    "Error inserting word '{0}' at location {1}", word, pos));
            }
        }

        /// <summary>
        /// Deletes all locations from the Location table for the specified word in this document.
        /// </summary>
        ///
        /// <param name="word">
        /// The word whose locations to remove.
        /// </param>
        ///
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while deleting the location.
        /// </exception>
        private void DbDeleteLocationByWordIdAndDocId(string word)
        {
            try
            {
                InitTransaction();

                // get word_id
                // SELECT Word_Id FROM Word WHERE Word = ?
                object wordId = sqlSelectWord.ExecuteScalar(connection, transaction, word);
                if (wordId == null)
                {
                    // word doesn't exist, nothing to do
                    return;
                }

                // DELETE FROM Location
                // WHERE Word_Id = ? AND Document_Id = ?
                sqlDeleteLocationByWordIdAndDocId.ExecuteNonQuery(connection, transaction,
                    wordId, parameters.DocumentID);
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, string.Format(
                    "Error deleting location for word '{0}'", word));
            }
        }

        /// <summary>
        /// Deletes all locations from the Location table for this document.
        /// </summary>
        ///
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while deleting the location.
        /// </exception>
        private void DbDeleteLocationByDocId()
        {
            try
            {
                InitTransaction();

                // DELETE FROM Location
                // WHERE Document_Id = ?
                sqlDeleteLocationByDocId.ExecuteNonQuery(connection, transaction,
                    parameters.DocumentID);
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, string.Format(
                    "Error deleting location for document id '{0}'", parameters.DocumentID));
            }
        }

        /// <summary>
        /// Updates the CompareOptions and Culture in the Document table.
        /// </summary>
        ///
        /// <param name="newCompareOptions">
        /// The new compare options.
        /// </param>
        /// <param name="newCultureInfo">
        /// The new culture.
        /// </param>
        ///
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while updating the document.
        /// </exception>
        private void DbUpdateCompareOptionsAndCulture(
            CompareOptions newCompareOptions, CultureInfo newCultureInfo)
        {
            try
            {
                InitTransaction();

                // UPDATE Document
                // SET Compare_Options = ?, Culture = ?
                // WHERE Document_Id = ?
                sqlUpdateDocument.ExecuteNonQuery(connection, transaction,
                    newCompareOptions.ToString(), newCultureInfo.Name, parameters.DocumentID);
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, "Error updating document's CompareOptions/Culture");
            }
        }

        /// <summary>
        /// Loads all document words from the Word/Location table, and build indices IDictionary.
        /// This will also load the document's CompareOptions and Culture settings from the Document table.
        /// </summary>
        ///
        /// <exception cref="LoadIndexDataException">
        /// Any error occurs while loading the tables.
        /// </exception>
        private void DbLoad()
        {
            try
            {
                InitTransaction();
                words.Clear();

                // get document CompareOptions and Culture settings
                // SELECT Compare_Options, Culture FROM Document
                // WHERE Document_Id = ?
                using (IDataReader reader = sqlSelectDocument.ExecuteReader(
                           connection, transaction, parameters.DocumentID))
                {
                    if (reader.Read())
                    {
                        compareOptions = (CompareOptions) CompareOptions.Parse(
                            typeof(CompareOptions), reader.GetString(0).Trim());
                        culture = new CultureInfo(reader.GetString(1).Trim());
                    }
                    else
                    {
                        // reset to default
                        compareOptions = Helper.DefaultCompareOptions;
                        culture = new CultureInfo(Helper.DefaultCulture);
                    }
                }

                // get all words and its locations
                // SELECT Word, Location FROM Location INNER JOIN Word ON Location.Word_Id = Word.Word_Id
                // WHERE Document_Id = ?
                using (IDataReader reader = sqlSelectWordLocation.ExecuteReader(
                    connection, transaction, parameters.DocumentID))
                {
                    while (reader.Read())
                    {
                        // get the word
                        string key = reader.GetString(0).Trim();
                        long pos = reader.GetInt64(1);

                        IList posList = (IList) words[key];
                        // add to words IDictionary
                        if (posList == null)
                        {
                            // create a new list and store it
                            posList = new ArrayList();
                            words[key] = posList;
                        }
                        posList.Add(pos);
                    }
                }
            }
            catch (Exception e)
            {
                throw new LoadIndexDataException(parameters, e, "Error loading data from Word/Location tables");
            }
            finally
            {
                // since no data is modified, we can safely rollback the opened transaction
                // and closes connection
                RollsbackTransaction();
            }
        }

        /// <summary>
        /// Lazily begins transaction if one is not started yet.
        /// This method should be called before any db operations are performed
        /// although only the first call will actually open the connection and starts transaction.
        /// </summary>
        private void InitTransaction()
        {
            if (transaction == null)
            {
                connection.Open();
                transaction = connection.BeginTransaction();
            }
        }

        /// <summary>
        /// Rolls back any pending transaction and closes the db connection.
        /// </summary>
        private void RollsbackTransaction()
        {
            if (transaction != null)
            {
                transaction.Rollback();
                transaction = null;
            }
            if (connection.State != ConnectionState.Closed)
            {
                connection.Close();
            }
        }
    }
}
