/*
 * SqlStatement.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Data;
using System.Data.Odbc;

namespace TopCoder.Document.Index.Persistence.DB
{
    /// <summary>
    /// A helper class to execute an Sql command on the database.
    /// </summary>
    ///
    /// <remarks>
    /// This is used to simplify main codes (particularly for dealing with Sql parameters)
    /// and enhanced performance by preparing (reusing) Sql command.
    /// Currently on Odbc related commands are supported, although adding support
    /// for other methods can be simply done by modifying InitCommand(), particularly
    /// the parameter handling.
    /// This class is not thread-safe. The internal IDbCommand is lazily instantiated
    /// on first execution. It's recommended that the instance is always executed
    /// with the same connection.
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    internal sealed class SqlStatement : IDisposable
    {
        /// <summary>
        /// The Sql statement to execute. Initialized in ctor() and never changed later.
        /// </summary>
        private readonly string sql;

        /// <summary>
        /// The Sql parameters name. Initialized in ctor() and never changed later.
        /// </summary>
        private readonly string[] parameters;

        /// <summary>
        /// The Sql command. Initialized on first call to any ExecuteXXX() methods
        /// and never changed afterward.
        /// </summary>
        private IDbCommand cmd;

        /// <summary>
        /// Tracks whether Dispose has been called.
        /// </summary>
        private bool isDisposed = false;

        /// <summary>
        /// Initializes a new instance of the <see cref="SqlStatement"/> class
        /// with the specified Sql command.
        /// </summary>
        ///
        /// <param name="commandText">
        /// The sql command which is represented by this class.
        /// </param>
        /// <param name="parameters">
        /// The sql parameter names
        /// (This might be empty for Odbc, but the count is still important)
        /// </param>
        internal SqlStatement(string commandText, params string[] parameters)
        {
            sql = commandText;
            this.parameters = parameters;
        }

        /// <summary>
        /// Executes a non-query sql.
        /// </summary>
        ///
        /// <param name="conn">
        /// The database connection to use.
        /// </param>
        /// <param name="transaction">
        /// The transaction to use.
        /// </param>
        /// <param name="paramValues">
        /// Any sql parameter values.
        /// </param>
        ///
        /// <returns>
        /// Number of rows affected.
        /// </returns>
        internal int ExecuteNonQuery(IDbConnection conn, IDbTransaction transaction, params object[] paramValues)
        {
            InitCommand(conn, transaction, paramValues);
            return cmd.ExecuteNonQuery();
        }

        /// <summary>
        /// Executes the sql and returns an <see cref="IDataReader"/>.
        /// </summary>
        ///
        /// <param name="conn">
        /// The database connection to use.
        /// </param>
        /// <param name="transaction">
        /// The transaction to use.
        /// </param>
        /// <param name="paramValues">
        /// Any sql parameter values.
        /// </param>
        ///
        /// <returns>
        /// The <see cref="IDataReader"/> representing for the sql query result.
        /// Caller should ensure the returned data reader is disposed of.
        /// </returns>
        internal IDataReader ExecuteReader(
            IDbConnection conn, IDbTransaction transaction, params object[] paramValues)
        {
            InitCommand(conn, transaction, paramValues);
            return cmd.ExecuteReader();
        }

        /// <summary>
        /// Executes the sql and returns the first column of the first row.
        /// </summary>
        ///
        /// <param name="conn">
        /// The database connection to use.
        /// </param>
        /// <param name="transaction">
        /// The transaction to use.
        /// </param>
        /// <param name="paramValues">
        /// Any sql parameter values.
        /// </param>
        ///
        /// <returns>
        /// The first column of the first row.
        /// </returns>
        internal object ExecuteScalar(IDbConnection conn, IDbTransaction transaction, params object[] paramValues)
        {
            InitCommand(conn, transaction, paramValues);
            return cmd.ExecuteScalar();
        }

        /// <summary>
        /// Disposes the internal IDbCommand.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            // tell GC not to run finalizer
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Finalizers which is called by the garbage collector.
        /// </summary>
        ~SqlStatement()
        {
            // Do not re-create Dispose clean-up code here.
            // Calling Dispose(false) is optimal in terms of
            // readability and maintainability.
            Dispose(false);
        }

        /// <summary>
        /// Implements the IDisposable pattern as specified in the MSDN documentation.
        /// </summary>
        ///
        /// <param name="disposing">
        /// true if this is called by user (managed resource can be referenced),
        /// false if this is called by garbage collector (don't touch managed resource).
        /// </param>
        private void Dispose(bool disposing)
        {
            if (!isDisposed)
            {
                // can only reference managed resource if user ask it
                if (disposing)
                {
                    // dispose the IDbCommand
                    if (cmd != null)
                    {
                        cmd.Dispose();
                    }
                }
            }
            isDisposed = true;
        }

        /// <summary>
        /// Prepares the IDbCommand instance to use and fills up the parameters.
        /// This method should be called by all ExecuteXXX() method,
        /// although only first invocation will actually initialize and prepare the command.
        /// </summary>
        ///
        /// <param name="conn">
        /// The db connection to use.
        /// </param>
        /// <param name="transaction">
        /// The transaction to use.
        /// </param>
        /// <param name="paramValues">
        /// The sql parameter values.
        /// </param>
        private void InitCommand(IDbConnection conn, IDbTransaction transaction, object[] paramValues)
        {
            // lazily instantiate the IDbCommand
            if (cmd == null)
            {
                // initialize cmd with the sql command string
                cmd = conn.CreateCommand();
                cmd.CommandText = sql;

                // create parameters and prepare it
                for (int i = 0; i < parameters.Length; i++)
                {
                    IDbDataParameter param = cmd.CreateParameter();
                    param.ParameterName = parameters[i];

                    cmd.Parameters.Add(param);
                }
                cmd.Prepare();
            }

            // always reinitialize the connection and transaction
            cmd.Connection = conn;
            cmd.Transaction = transaction;

            // fills-up the parameters
            for (int i = 0; i < paramValues.Length; i++)
            {
                // unfortunately there's no way to assign a parameter value via IDbDataParameter
                // so we have to cast it to the concrete classes
                if (cmd.Parameters[i] is OdbcParameter)
                {
                    ((OdbcParameter) cmd.Parameters[i]).Value = paramValues[i];
                }
            }
        }
    }
}
