/*
 * XmlCollectionIndexSource.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Ghostar, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Xml;

namespace TopCoder.Document.Index.Persistence.Xml
{
    /// <summary>
    /// Implements the <see cref="ICollectionIndexSource"/> to provide the collection index sources
    /// in an Xml document.
    /// </summary>
    ///
    /// <remarks>
    /// This implementation saves and parses files according to the Xml Schema below.
    /// This class provides functionality for getting information about a particular document,
    /// as well as saving back out all documents in the collection.
    /// Note that <see cref="GetEnumerator"/> returns an <see cref="IDictionaryEnumerator"/>
    /// which is not thread-safe, but a <see cref="SyncRoot"/> is provided.
    /// </remarks>
    ///
    /// <example>
    /// Here's a sample xml file: <code><![CDATA[
    /// <?xml version="1.0" encoding="utf-8"?>
    /// <Documents>
    ///     <Document>
    ///         <Name>Document 0</Name>
    ///         <FilePath>..\..\test_files\Document0.xml</FilePath>
    ///     </Document>
    ///     <Document>
    ///         <Name>Document 1</Name>
    ///         <FilePath>..\..\test_files\Document1.xml</FilePath>
    ///     </Document>
    ///     <Document>
    ///         <Name>Document 2</Name>
    ///         <FilePath>..\..\test_files\Document2.xml</FilePath>
    ///     </Document>
    /// </Documents>
    /// ]]></code>
    /// Here's a sample code: <code><![CDATA[
    /// // load the collection from an xml file
    /// ICollectionIndexSource xmlSource = new XmlCollectionIndexSource();
    /// IParameters parameters = new SimpleIndexParameters(colFile);
    /// xmlSource.Load(parameters);
    ///
    /// // enumerate through indices
    /// IDictionaryEnumerator enumerator = xmlSource.GetEnumerator();
    /// int count = 0;
    /// while (enumerator.MoveNext())
    /// {
    ///     count++;
    /// }
    ///
    /// // add a new index
    /// IParameters documentParams = new SimpleIndexParameters(docFile);
    /// IndexInfo info = new IndexInfo(typeof(XmlDocumentIndexSource).Assembly.GetName().Name,
    ///     typeof(XmlDocumentIndexSource).FullName, documentParams);
    /// xmlSource.Add("New document", info);
    ///
    /// // add using indexer
    /// xmlSource["New document"] = info;
    ///
    /// // save the changes
    /// xmlSource.Save();
    ///
    /// // check index exists
    /// bool exists = xmlSource.Contains("New document");
    ///
    /// // remove the index
    /// xmlSource.Remove("New document");
    ///
    /// // use indexer
    /// IndexInfo retrieved = xmlSource["Document 2"];
    ///
    /// // clear all indices
    /// xmlSource.Clear();
    ///
    /// // get collection file path
    /// string file = ((XmlCollectionIndexSource) xmlSource).FilePath
    ///
    /// // save the changes
    /// xmlSource.Save();
    /// ]]></code>
    /// </example>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class XmlCollectionIndexSource : ICollectionIndexSource
    {
        /// <summary>
        /// The Xml Schema for a documents xml file.
        /// Used to validate the loaded xml.
        /// </summary>
        private const string DocumentsXmlSchema =
            @"<?xml version=""1.0"" encoding=""UTF-8""?>"
            + @"<xs:schema xmlns:xs=""http://www.w3.org/2001/XMLSchema"">"
                + @"<xs:element name=""Documents"">"
                    + @"<xs:complexType>"
                        + @"<xs:sequence>"
                            + @"<xs:element name=""Document"" type=""Document"" "
                                + @"maxOccurs=""unbounded"" minOccurs=""0"" />"
                        + @"</xs:sequence>"
                    + @"</xs:complexType>"
                + @"</xs:element>"

                + @"<xs:complexType name=""Document"">"
                    + @"<xs:sequence>"
                        + @"<xs:element name=""Name"" type=""xs:string"" />"
                        + @"<xs:element name=""FilePath"" type=""xs:string"" />"
                    + @"</xs:sequence>"
                + @"</xs:complexType>"
            + @"</xs:schema>";

        /// <summary>
        /// Root xml element tag.
        /// </summary>
        private const string DocumentsElement = "Documents";

        /// <summary>
        /// Xml element tag for a document index.
        /// </summary>
        private const string DocumentElement = "Document";

        /// <summary>
        /// Xml element tag for the index name.
        /// </summary>
        private const string NameElement = "Name";

        /// <summary>
        /// Xml element tag for the file path.
        /// </summary>
        private const string FilePathElement = "FilePath";

        /// <summary>
        /// XPath to select all Document nodes: /Documents/Document.
        /// </summary>
        private const string XPathSelectDocument = "/" + DocumentsElement + "/" + DocumentElement;

        /// <summary>
        /// The type to populate IndexInfo (used in ParseXmlDoc).
        /// </summary>
        private static readonly Type XmlDocumentIndexSourceType = typeof(XmlDocumentIndexSource);

        /// <summary>
        /// This member variable holds the loaded names and document indices
        /// loaded from the XML file that this instance is working against.
        /// The keys in this dictionary are the names, given by the user,
        /// of each document index, and the values are the IndexInfo instances,
        /// created in the Load method, that can be accessed by the user.
        /// This collection is accessed using the accessors in this class,
        /// and it is loaded in the Load methods.
        /// </summary>
        private IDictionary indices = SortedList.Synchronized(new SortedList());

        /// <summary>
        /// This member variable holds the path to the file where XML is loaded from and saved to.
        /// This member variable can be set in the constructor, and
        /// it can be set and retrieved using the FilePath property.
        /// </summary>
        private string filePath;

        /// <summary>
        /// Gets the object to use when synchronizing access to the indices collection.
        /// </summary>
        ///
        /// <value>
        /// The object to use when synchronizing access to the indices collection.
        /// </value>
        public object SyncRoot
        {
            get
            {
                return indices.SyncRoot;
            }
        }

        /// <summary>
        /// Gets or sets the file path where XML is loaded from and saved to.
        /// </summary>
        ///
        /// <value>
        /// The file path where XML is loaded from and saved to.
        /// </value>
        ///
        /// <exception cref="ArgumentNullException">
        /// Setting to a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Setting to an empty trimmed string or the file path is invalid
        /// (either it is malformed or it points to a directory).
        /// </exception>
        public string FilePath
        {
            get
            {
                return filePath;
            }
            set
            {
                Helper.ValidateFilePath(value, "FilePath");
                filePath = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlCollectionIndexSource"/>.
        /// </summary>
        ///
        /// <remarks>
        /// The instance will initially be blank.
        /// The <see cref="Load"/> method should be used to load the actual data in
        /// for this class.
        /// </remarks>
        public XmlCollectionIndexSource()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlCollectionIndexSource"/> class.
        /// </summary>
        ///
        /// <param name="filePath">
        /// The file path to save to.
        /// </param>
        ///
        /// <remarks>
        /// Note that <see cref="Load"/> method should be used to load the actual data in
        /// for this class.
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="filePath"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="filePath"/> is an empty trimmed string or is invalid
        /// (either it is malformed or it points to a directory).
        /// </exception>
        public XmlCollectionIndexSource(string filePath)
        {
            // setter will validate the file path
            FilePath = filePath;
        }

        /// <summary>
        /// Gets or sets the <see cref="IndexInfo"/> for the given collection index name.
        /// </summary>
        ///
        /// <param name="indexName">
        /// Name of the index to return or set the <see cref="IndexInfo"/> for.
        /// </param>
        ///
        /// <value>
        /// The <see cref="IndexInfo"/> for the given collection index name or
        /// null if no <see cref="IndexInfo"/> for the given name exists in the collection.
        /// </value>
        ///
        /// <remarks>
        /// Note that if an <see cref="IndexInfo"/> already exists for the specified name,
        /// it will be replaced.
        /// The index name will be trimmed before stored.
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="indexName"/> is a null reference or setting to a null value.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="indexName"/> is an empty trimmed string or
        /// the IndexInfo is invalid (eg. the assembly/class name must be
        /// <see cref="XmlDocumentIndexSource"/> and the parameter must be
        /// of type <see cref="SimpleIndexParameters"/>.
        /// </exception>
        public IndexInfo this[string indexName]
        {
            get
            {
                if (indexName == null)
                {
                    throw new ArgumentNullException("indexName", "'indexName' cannot be null");
                }
                if (indexName.Trim().Length == 0)
                {
                    throw new ArgumentException("'indexName' cannot be an empty trimmed string", "indexName");
                }

                return (IndexInfo) indices[indexName];
            }
            set
            {
                if (indexName == null)
                {
                    throw new ArgumentNullException("indexName", "'indexName' cannot be null");
                }
                string key = indexName.Trim();
                if (key.Length == 0)
                {
                    throw new ArgumentException("'indexName' cannot be an empty trimmed string", "indexName");
                }
                if (value == null)
                {
                    throw new ArgumentNullException("indexInfo", "'indexInfo' cannot be null");
                }
                Helper.ValidateIndexInfo(value, XmlDocumentIndexSourceType, typeof(SimpleIndexParameters));

                indices[key] = value;
            }
        }

        /// <summary>
        /// Adds the given <see cref="IndexInfo"/> which is associated with the specified index name.
        /// </summary>
        ///
        /// <param name="indexName">
        /// The name of the index to add.
        /// </param>
        /// <param name="indexInfo">
        /// The <see cref="IndexInfo"/> to add with the given name.
        /// </param>
        ///
        /// <remarks>
        /// The index name will be trimmed before stored.
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="indexName"/> is an empty trimmed string or
        /// if a value for the specified index name already exists or
        /// the <paramref name="indexInfo"/> is invalid (eg. the assembly/class name must be
        /// <see cref="XmlDocumentIndexSource"/> and the parameter must be
        /// of type <see cref="SimpleIndexParameters"/>.
        /// </exception>
        public void Add(string indexName, IndexInfo indexInfo)
        {
            if (indexName == null)
            {
                throw new ArgumentNullException("indexName", "'indexName' cannot be null");
            }
            string key = indexName.Trim();
            if (key.Length == 0)
            {
                throw new ArgumentException("'indexName' cannot be an empty trimmed string", "indexName");
            }
            if (indexInfo == null)
            {
                throw new ArgumentNullException("indexInfo", "'indexInfo' cannot be null");
            }
            Helper.ValidateIndexInfo(indexInfo, XmlDocumentIndexSourceType, typeof(SimpleIndexParameters));

            // will throw AE if indexName already exists
            indices.Add(key, indexInfo);
        }

        /// <summary>
        /// Removes the specified collection index name.
        /// </summary>
        ///
        /// <remarks>
        /// Nothing will occur if the specified name doesn't exist.
        /// </remarks>
        ///
        /// <param name="indexName">
        /// Name of the index to remove.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="indexName"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="indexName"/> is an empty trimmed string.
        /// </exception>
        public void Remove(string indexName)
        {
            if (indexName == null)
            {
                throw new ArgumentNullException("indexName", "'indexName' cannot be null");
            }
            if (indexName.Trim().Length == 0)
            {
                throw new ArgumentException("'indexName' cannot be an empty trimmed string", "indexName");
            }

            indices.Remove(indexName);
        }

        /// <summary>
        /// Determines whether an <see cref="IndexInfo"/> exists for the given name.
        /// </summary>
        ///
        /// <param name="indexName">
        /// The index name to look for.
        /// </param>
        ///
        /// <returns>
        /// true if the collection contains the given index; otherwise, false.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="indexName"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="indexName"/> is an empty trimmed string.
        /// </exception>
        public bool Contains(string indexName)
        {
            if (indexName == null)
            {
                throw new ArgumentNullException("indexName", "'indexName' cannot be null");
            }
            if (indexName.Trim().Length == 0)
            {
                throw new ArgumentException("'indexName' cannot be an empty trimmed string", "indexName");
            }

            return indices.Contains(indexName);
        }

        /// <summary>
        /// Clears all document indices.
        /// </summary>
        public void Clear()
        {
            indices.Clear();
        }

        /// <summary>
        /// Returns the enumerator for the indices collection, which can be used by the user
        /// to enumerate through all document indices in this collection.
        /// </summary>
        ///
        /// <returns>
        /// The enumerator for all <see cref="IndexInfo"/> in the collection.
        /// </returns>
        public IDictionaryEnumerator GetEnumerator()
        {
            return indices.GetEnumerator();
        }

        /// <summary>
        /// Loads all document index names and its <see cref="IndexInfo"/> instances from the given
        /// Xml file path, in the <see cref="SimpleIndexParameters"/> instance provided.
        /// </summary>
        ///
        /// <remarks>
        /// The Xml in the file path given should conform the Documents schema outlined
        /// in the class documentation.
        /// The associated <see cref="IndexInfo"/> will always refer to <see cref="XmlDocumentIndexSource"/>.
        /// This method will also updates <see cref="FilePath"/> as specified in the parameter.
        /// If any exception occurs, existing (loaded) indices will stay as-is;
        /// otherwise, existing indices is cleared first.
        /// </remarks>
        ///
        /// <param name="parameters">
        /// The <see cref="SimpleIndexParameters"/> containing the xml file path to load.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="parameters"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="parameters"/> contains an empty trimmed path or
        /// is not a <see cref="SimpleIndexParameters"/> instance or
        /// contains invalid FileName parameters
        /// (malformed, points to a directory, doesn't exists).
        /// </exception>
        /// <exception cref="LoadIndexDataException">
        /// Any errors occur while loading the Xml.
        /// </exception>
        public void Load(IParameters parameters)
        {
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters", "'parameters' cannot be null");
            }
            if (!(parameters is SimpleIndexParameters))
            {
                throw new ArgumentException("'parameters' must be of SimpleIndexParameters type", "parameters");
            }

            // filepath must be valid (and save it to member var)
            FilePath = ((SimpleIndexParameters) parameters).FileName;
            // file not exists, throw AE
            if (!(File.Exists(FilePath)))
            {
                throw new ArgumentException(string.Format(
                    "FilePath {0} doesn't exist", FilePath));
            }

            // load the xml file and parse it (build the indices IDictionary)
            XmlDocument doc = LoadAndValidateXml(parameters);
            ParseXmlDoc(parameters, doc);
        }

        /// <summary>
        /// Saves all document index names and <see cref="IndexInfo"/> instances corresponding to those names,
        /// to the given Xml file path.
        /// </summary>
        ///
        /// <exception cref="InvalidOperationException">
        /// <see cref="FilePath"/> is a null reference.
        /// </exception>
        /// <exception cref="SaveIndexDataException">
        /// Any errors occur while saving the Xml.
        /// </exception>
        public void Save()
        {
            if (FilePath == null)
            {
                throw new InvalidOperationException("'FilePath' is a null reference");
            }

            try
            {
                XmlTextWriter w = null;
                try
                {
                    // instead of XmlDocument, XmlTextWriter provides faster, forward-only xml writing
                    w = new XmlTextWriter(FilePath, new UTF8Encoding());

                    // use the indented formatter, adhere to our coding standard of 4 spaces
                    w.Formatting = Formatting.Indented;
                    w.Indentation = Helper.XmlIndentation;

                    // write the XML declaration with version 1.0
                    w.WriteStartDocument();

                    // lv 1: <Documents>
                    w.WriteStartElement(DocumentsElement);

                    // write each element
                    foreach (DictionaryEntry idx in indices)
                    {
                        // lv 2: <Document>
                        w.WriteStartElement(DocumentElement);

                        // lv 3: <Name>...</Name>
                        w.WriteElementString(NameElement, (string) idx.Key);

                        // lv 3: <FilePath>...</FilePath>
                        IParameters param = ((IndexInfo) idx.Value).Parameters;
                        w.WriteElementString(FilePathElement, ((SimpleIndexParameters) param).FileName);

                        // lv 2: </Document>
                        w.WriteEndElement();
                    }

                    // lv 1: </Documents>
                    w.WriteEndElement();
                }
                finally
                {
                    if (w != null)
                    {
                        // this may throw Exception, which will be reencapsulated into SaveIndexDataException
                        w.Close();
                    }
                }
            }
            catch (SaveIndexDataException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, string.Format(
                    "Error saving to '{0}'", FilePath));
            }
        }

        /// <summary>
        /// Loads and validates FilePath into an XmlDocument.
        /// </summary>
        ///
        /// <remarks>
        /// The file is loaded using <see cref="XmlValidatingReader"/> using a schema.
        /// The schema will provide xml structure validation.
        /// </remarks>
        ///
        /// <param name="param">
        /// The original <see cref="IParameters"/> used only for exception parameter.
        /// </param>
        ///
        /// <returns>
        /// A validated <see cref="XmlDocument"/>.
        /// </returns>
        ///
        /// <exception cref="LoadIndexDataException">
        /// Error loading and validating the xml file.
        /// </exception>
        private XmlDocument LoadAndValidateXml(IParameters param)
        {
            try
            {
                return Helper.LoadAndValidateXml(FilePath, DocumentsXmlSchema);
            }
            catch (Exception e)
            {
                // provide a good error message from the XmlValidatingReader
                throw new LoadIndexDataException(param, e, string.Format(
                    "Error loading and validating xml file '{0}': {1}", FilePath, e.Message));
            }
        }

        /// <summary>
        /// Parses the specified XmlDocument and build the indices IDictionary.
        /// </summary>
        ///
        /// <param name="param">
        /// The original <see cref="IParameters"/> used only for exception parameter.
        /// </param>
        /// <param name="xmlDoc">
        /// The <see cref="XmlDocument"/> to parse.
        /// </param>
        ///
        /// <exception cref="LoadIndexDataException">
        /// Error parsing the xml file.
        /// </exception>
        private void ParseXmlDoc(IParameters param, XmlDocument xmlDoc)
        {
            // save old indices first (rollback to this when any error occurs)
            IDictionary oldIndices = indices;
            try
            {
                // clear indices by creating a new one
                indices = SortedList.Synchronized(new SortedList());

                // select all Document nodes, and process each
                XmlNodeList docs = xmlDoc.SelectNodes(XPathSelectDocument);
                foreach (XmlNode doc in docs)
                {
                    // get Name and FilePath
                    string name = doc.SelectSingleNode(NameElement).InnerText.Trim();
                    string filePath = doc.SelectSingleNode(FilePathElement).InnerText.Trim();

                    // prepare IndexInfo and add to indices IDictionary
                    IParameters idxParam = new SimpleIndexParameters(filePath);
                    IndexInfo idxInfo = new IndexInfo(
                        XmlDocumentIndexSourceType.Assembly.FullName,
                        XmlDocumentIndexSourceType.FullName,
                        idxParam);
                    Add(name, idxInfo);
                }
            }
            catch (Exception e)
            {
                // roll-back (so we're not left with half-loaded data)
                indices = oldIndices;
                throw new LoadIndexDataException(param, e, string.Format(
                    "Error parsing xml file '{0}'", FilePath));
            }
        }
    }
}
