/*
 * TestDBDocumentIndexParameters.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using NUnit.Framework;

namespace TopCoder.Document.Index.Persistence.DB.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="DBDocumentIndexParameters"/> class.
    /// </summary>
    [TestFixture]
    public class TestDBDocumentIndexParameters
    {
        /// <summary>
        /// The <see cref="DBDocumentIndexParameters"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private DBDocumentIndexParameters param;

        /// <summary>
        /// A sample connection string for test.
        /// </summary>
        private const string conn = "TopCoder DB";

        /// <summary>
        /// A sample document id for test.
        /// </summary>
        private const int docId = 1234567890;

        /// <summary>
        /// Initializes the <see cref="DBDocumentIndexParameters"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            param = new DBDocumentIndexParameters(conn, docId);
        }

        /// <summary>
        /// Tests constructor(string, long) accuracy
        /// via ConnectionName and DocumentID property getters.
        /// </summary>
        [Test]
        public void TestCtorConnectionNameDocumentIDGetter()
        {
            Assert.AreEqual(conn, param.ConnectionName, "The ConnectionName is not initialized");
            Assert.AreEqual(docId, param.DocumentID, "The DocumentID is not initialized");
        }

        /// <summary>
        /// Tests constructor(string, long) accuracy given document id = 0.
        /// Asserts via ConnectionName and DocumentID property getters.
        /// </summary>
        [Test]
        public void TestCtorConnectionNameDocumentIDGetterZero()
        {
            param = new DBDocumentIndexParameters(conn, 0);
            Assert.AreEqual(conn, param.ConnectionName, "The ConnectionName is not initialized");
            Assert.AreEqual(0, param.DocumentID, "The DocumentID is 0");
        }

        /// <summary>
        /// Tests constructor(string, long) given null connection name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorNullConnectionName()
        {
            new DBDocumentIndexParameters(null, docId);
        }

        /// <summary>
        /// Tests constructor(string, string) given empty string connection name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringEmpty()
        {
            new DBDocumentIndexParameters(" ", docId);
        }

        /// <summary>
        /// Tests constructor(string, long) given negative document id.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorNegativeId()
        {
            new DBDocumentIndexParameters(conn, -1);
        }

        /// <summary>
        /// Tests GetHashCode accuracy, that it returns consistent value according to connection name
        /// and document id.
        /// </summary>
        [Test]
        public void TestGetHashCode()
        {
            Assert.AreEqual(param.GetHashCode(), param.GetHashCode(), "Same hashcode for same instance");

            // create a new instance with same connection string and different document id
            DBDocumentIndexParameters param2 = new DBDocumentIndexParameters(conn, docId + 1);
            Assert.IsTrue(param.GetHashCode() != param2.GetHashCode(), "Hashcode should be different");
        }

        /// <summary>
        /// Tests Equals accuracy, that it returns true given instance
        /// with the same connection name and document id
        /// and false for different document id.
        /// </summary>
        [Test]
        public void TestEquals()
        {
            // create a new instance with equal connection string and document id
            DBDocumentIndexParameters param2 = new DBDocumentIndexParameters(conn, docId);
            Assert.IsTrue(param.Equals(param2), "Same connection string and id");

            // test with different id
            Assert.IsFalse(param.Equals(new DBDocumentIndexParameters(conn, docId + 1)),
                "Different connection string");
        }

        /// <summary>
        /// Tests Equals accuracy, that it returns false given instance
        /// with the different connection name (case-sensitive).
        /// </summary>
        [Test]
        public void TestEqualsCaseSensitivity()
        {
            // create a new instance with lowercase connection string
            DBDocumentIndexParameters param2 = new DBDocumentIndexParameters(conn.ToLower(), docId);
            Assert.IsFalse(param.Equals(param2), "Different connection string");
        }

        /// <summary>
        /// Tests Equals given null argument. Expecting false is returned.
        /// </summary>
        [Test]
        public void TestEqualsNull()
        {
            Assert.IsFalse(param.Equals(null), "Null is different");
        }

        /// <summary>
        /// Tests Equals given non DBDocumentIndexParameters argument.
        /// Expecting false is returned.
        /// </summary>
        [Test]
        public void TestEqualsNonDBDocumentIndexParameters()
        {
            Assert.IsFalse(param.Equals(123), "Non DBDocumentIndexParameters");
        }
    }
}
