/*
 * TestDBPersistenceConfigurationException.cs
 *
 * Product Licensing
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using NUnit.Framework;

namespace TopCoder.Document.Index.Persistence.DB.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="DBPersistenceConfigurationException"/> class.
    /// </summary>
    [TestFixture]
    public class TestDBPersistenceConfigurationException
    {
        /// <summary>
        /// A sample error message to use.
        /// </summary>
        private const string errMessage = "Error configuring db persistence";

        /// <summary>
        /// A sample inner exception to use.
        /// </summary>
        private static Exception inner = new Exception();

        /// <summary>
        /// Tests constructor(string) accuracy.
        /// </summary>
        [Test]
        public void TestCtorString()
        {
            // instantiate with a sample error message
            DBPersistenceConfigurationException e = new DBPersistenceConfigurationException(errMessage);

            // assert error message is correct, and inner exception is null
            Assert.AreEqual(errMessage, e.Message, "The error message is incorrect");
            Assert.IsNull(e.InnerException, "Inner exception should be null");
        }

        /// <summary>
        /// Tests constructor(string) with a null error message.
        /// Expecting a default error message in the exception.
        /// </summary>
        [Test]
        public void TestCtorStringNull()
        {
            // instantiate with a null error message
            DBPersistenceConfigurationException e = new DBPersistenceConfigurationException(null);

            // assert error message is initialized to default message and inner exception is null
            Assert.IsNotNull(e.Message, "The error message is the default message");
            Assert.IsNull(e.InnerException, "Inner exception should be null");
        }

        /// <summary>
        /// Tests constructor(Type, string) with an empty error message.
        /// Expecting an empty error message in the exception.
        /// </summary>
        [Test]
        public void TestCtorStringEmpty()
        {
            // instantiate with an empty error message
            DBPersistenceConfigurationException e = new DBPersistenceConfigurationException(" ");

            // assert error message is empty and inner exception is null
            Assert.AreEqual(" ", e.Message, "The error message should be empty");
            Assert.IsNull(e.InnerException, "Inner exception should be null");
        }

        /// <summary>
        /// Tests constructor(string, Exception) accuracy.
        /// </summary>
        [Test]
        public void TestCtorStringException()
        {
            // instantiate with a sample error message and inner exception
            DBPersistenceConfigurationException e = new DBPersistenceConfigurationException(errMessage, inner);

            // assert error message and inner exception are correct
            Assert.AreEqual(errMessage, e.Message, "The error message is incorrect");
            Assert.AreSame(inner, e.InnerException, "Inner exception is different");
        }

        /// <summary>
        /// Tests constructor(string, Exception) with a null error message.
        /// Expecting a default error message in the exception.
        /// </summary>
        [Test]
        public void TestCtorStringExceptionNullString()
        {
            // instantiate with a null error message and inner exception
            DBPersistenceConfigurationException e = new DBPersistenceConfigurationException(null, inner);

            // assert error message is initialized to default message and inner exception are correct
            Assert.IsNotNull(e.Message, "The error message is the default message");
            Assert.AreSame(inner, e.InnerException, "Inner exception is different");
        }

        /// <summary>
        /// Tests constructor(string, Exception) with a null inner exception.
        /// Expecting a null inner exception.
        /// </summary>
        [Test]
        public void TestCtorStringExceptionNullInner()
        {
            // instantiate with a sample error message and a null inner exception
            DBPersistenceConfigurationException e = new DBPersistenceConfigurationException(errMessage, null);

            // assert error message is correct and inner exception is null
            Assert.AreEqual(errMessage, e.Message, "The error message is incorrect");
            Assert.IsNull(e.InnerException, "Inner exception should be null");
        }

        /// <summary>
        /// Tests constructor(string, Exception) with an empty error message.
        /// Expecting an empty error message in the exception.
        /// </summary>
        [Test]
        public void TestCtorStringExceptionEmptyString()
        {
            // instantiate with an empty error message and inner exception
            DBPersistenceConfigurationException e = new DBPersistenceConfigurationException(" ", inner);

            // assert error message is empty and inner exception is correct
            Assert.AreEqual(" ", e.Message, "The error message should be empty");
            Assert.AreSame(inner, e.InnerException, "Inner exception is different");
        }

        /// <summary>
        /// Tests <see cref="DBPersistenceConfigurationException"/> inherits from <see cref="ApplicationException"/>.
        /// </summary>
        [Test]
        public void TestInheritance()
        {
            try
            {
                throw new DBPersistenceConfigurationException(errMessage);
            }
            catch (Exception e)
            {
                // ok only if it's a direct subclass of ApplicationException
                Assert.AreEqual(typeof(ApplicationException), e.GetType().BaseType,
                    "DBPersistenceConfigurationException is not a direct subclass of ApplicationException");
            }
        }
    }
}
