/*
 * TestDBPersistenceParameters.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using NUnit.Framework;

namespace TopCoder.Document.Index.Persistence.DB.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="DBPersistenceParameters"/> class.
    /// </summary>
    [TestFixture]
    public class TestDBPersistenceParameters
    {
        /// <summary>
        /// The <see cref="DBPersistenceParameters"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private DBPersistenceParameters param;

        /// <summary>
        /// A sample connection string for test.
        /// </summary>
        private const string conn = "TopCoder DB";

        /// <summary>
        /// Initializes the <see cref="DBPersistenceParameters"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            param = new DBPersistenceParameters(conn);
        }

        /// <summary>
        /// Tests constructor(string) accuracy via ConnectionName property getter.
        /// </summary>
        [Test]
        public void TestCtorConnectionNameGetter()
        {
            Assert.AreEqual(conn, param.ConnectionName, "The ConnectionName is not initialized");
        }

        /// <summary>
        /// Tests constructor(string) given null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorNull()
        {
            new DBPersistenceParameters(null);
        }

        /// <summary>
        /// Tests constructor(string) given empty string argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringEmpty()
        {
            new DBPersistenceParameters(" ");
        }

        /// <summary>
        /// Tests GetHashCode accuracy, that it returns consistent value according to connection name.
        /// </summary>
        [Test]
        public void TestGetHashCode()
        {
            Assert.AreEqual(param.GetHashCode(), param.GetHashCode(), "Same hashcode for same instance");

            // create a new instance with different connection string
            DBPersistenceParameters param2 = new DBPersistenceParameters(conn + conn);
            Assert.IsTrue(param.GetHashCode() != param2.GetHashCode(), "Hashcode should be different");
        }

        /// <summary>
        /// Tests Equals accuracy, that it returns true given instance with the same connection name
        /// and false for different connection name.
        /// </summary>
        [Test]
        public void TestEquals()
        {
            // create a new instance with equal connection string
            DBPersistenceParameters param2 = new DBPersistenceParameters(conn);
            Assert.IsTrue(param.Equals(param2), "Same connection string");

            // test with different connection name
            Assert.IsFalse(param.Equals(new DBPersistenceParameters(conn + conn)), "Different connection string");
        }

        /// <summary>
        /// Tests Equals accuracy, that it returns false given instance
        /// with the different connection name (case-sensitive).
        /// </summary>
        [Test]
        public void TestEqualsCaseSensitivity()
        {
            // create a new instance with lowercase connection string
            DBPersistenceParameters param2 = new DBPersistenceParameters(conn.ToLower());
            Assert.IsFalse(param.Equals(param2), "Different connection string");
        }

        /// <summary>
        /// Tests Equals given null argument. Expecting false is returned.
        /// </summary>
        [Test]
        public void TestEqualsNull()
        {
            Assert.IsFalse(param.Equals(null), "Null is different");
        }

        /// <summary>
        /// Tests Equals given non DBPersistenceParameters argument.
        /// Expecting false is returned.
        /// </summary>
        [Test]
        public void TestEqualsNonDBPersistenceParameters()
        {
            Assert.IsFalse(param.Equals(123), "Non DBPersistenceParameters");
        }
    }
}
