/*
 * DemoDocumentIndexer.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Data;
using System.IO;
using System.Collections;
using System.Globalization;
using NUnit.Framework;
using TopCoder.Data.ConnectionFactory;
using TopCoder.Document.Index.Persistence.DB;
using TopCoder.Document.Index.WordStreams;

namespace TopCoder.Document.Index.Persistence.Tests
{
    /// <summary>
    /// Demonstrates the component as specified in the Document Indexer component
    /// using the DB Persistence.
    /// </summary>
    [TestFixture]
    public class DemoDocumentIndexer : TestCommon
    {
        /// <summary>
        /// Sql to deletes all rows from the document/word/location table.
        /// </summary>
        private SqlStatement[] clearsData = new SqlStatement[]
            {
                new SqlStatement("DELETE FROM Location"),
                new SqlStatement("DELETE FROM Word"),
                new SqlStatement("DELETE FROM Document")
            };

        /// <summary>
        /// Loads configuration file and cleans-up environment.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // Connection Factory and DBDocumentIndexSource configs
            base.LoadConfigFile(ConnectionFactoryConfig,
                @"..\..\test_files\configs\DBDocumentIndexSource.xml");
            CleanUpEnvironment();
        }

        /// <summary>
        /// Cleans up environment and unload config.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            CleanUpEnvironment();
            base.LoadConfigFile();
        }

        /// <summary>
        /// Cleans up by deleting all rows from the database.
        /// </summary>
        public void CleanUpEnvironment()
        {
            // deletes all db records
            DeletesAllData();
        }

        /// <summary>
        /// Deletes all rows from the database.
        /// </summary>
        private void DeletesAllData()
        {
            IDbConnection conn = ConnectionManager.Instance.CreateDbConnection(
                DbConnectionName, new Hashtable());
            try
            {
                conn.Open();
                foreach (SqlStatement stmt in clearsData)
                {
                    stmt.ExecuteNonQuery(conn, null);
                }
            }
            finally
            {
                conn.Close();
            }
        }

        /// <summary>
        /// All kinds of function about dealing with collection document index.
        /// </summary>
        [Test]
        public void DealWithCollectionDocumentIndex()
        {
            ICollectionIndexSource indexSource = null;
            IDocumentIndexSource source1 = null;
            try
            {
                // Create collection index source.
                indexSource = new DBCollectionIndexSource();
                indexSource.Load(new DBPersistenceParameters(DbConnectionName));

                // Create collection index class and load all document indices.
                CollectionIndex index = new CollectionIndex(indexSource);

                // Clear all indices from collection.
                index.Clear();

                // Add some indices to collection.
                source1 = new DBDocumentIndexSource();
                source1.Load(new DBDocumentIndexParameters(DbConnectionName, 1));
                DocumentIndex doc1 = new DocumentIndex(source1);
                index.Add("txt", doc1);

                // Enumerate document that contains word "this".
                EvaluateCondition evaluationCondition = new EvaluateCondition(CollectionIndex.DocumentContainsWord);
                IDictionaryEnumerator enumerator = index.GetConditionalDocumentEnumerator(
                    evaluationCondition, "this");
                while(enumerator.MoveNext())
                {
                    // Get document index name.
                    string name = (string) enumerator.Key;

                    // Get the positions of "this".
                    DocumentIndex docIndex = (DocumentIndex) enumerator.Value;
                    IList position = docIndex["this"];
                }

                // Remove index from collection
                index.Remove("txt");

                // save collection index to persistent source.
                index.Save();
            }
            finally
            {
                if (source1 != null)
                {
                    ((IDisposable) source1).Dispose();
                }
                if (indexSource != null)
                {
                    ((IDisposable) indexSource).Dispose();
                }
            }
        }

        /// <summary>
        /// All kinds of function about dealing with single document index.
        /// </summary>
        [Test]
        public void DealWithSingleDocumentIndex()
        {
            // create a new document collection first
            IParameters documentParams = null;
            ICollectionIndexSource dbColl = null;
            try
            {
                dbColl = new DBCollectionIndexSource();
                IParameters colParams = new DBPersistenceParameters(DbConnectionName);
                dbColl.Load(colParams);

                // save and re-load to get the document params
                documentParams = new DBDocumentIndexParameters(DbConnectionName, 1);
                IndexInfo info = new IndexInfo(typeof(DBDocumentIndexSource).Assembly.GetName().Name,
                    typeof(DBDocumentIndexSource).FullName, documentParams);
                dbColl.Add("New document", info);
                dbColl.Save();
                dbColl.Load(colParams);
                documentParams = dbColl["New document"].Parameters;
            }
            finally
            {
                if (dbColl != null)
                {
                    ((IDisposable) dbColl).Dispose();
                }
            }

            // Create document index source (DB persistence)
            IDocumentIndexSource indexSource = null;
            IDocumentIndexSource indexSource2 = null;
            try
            {
                indexSource = new DBDocumentIndexSource();

                // Loading index data from that file.
                indexSource.Load(documentParams);

                // Create document index class.
                DocumentIndex simpleIndex = new DocumentIndex(indexSource);

                // Count for a specified word.
                try
                {
                    int thisCount = simpleIndex.GetCountOfWordOccurrences("Please");
                }
                catch (Exception)
                {
                    // this[] indexer should return empty list instead of null
                    // when word is not found
                }

                // Create document index source and let it output to the temp file when call Save.
                indexSource2 = new DBDocumentIndexSource();
                indexSource2.Load(documentParams);

                // Create document index class using the second source.
                DocumentIndex simpleIndex2 = new DocumentIndex(indexSource2);

                // Create word stream, which will be used to fill the document index.
                WordStream textFile = new TextFile(@"..\..\test_files\Notes.txt", null,
                    new CultureInfo("en-US"),  CompareOptions.IgnoreCase);

                // Fill index data using the word stream
                simpleIndex2.Create(textFile);

                // Test whether the file contains a specified word "on" ignoring case.
                bool containOn = simpleIndex2.Contains("on");
                Assert.IsFalse(containOn, "No 'on'");

                // count word occurence
                int pleaseCount = simpleIndex2.GetCountOfWordOccurrences("Please");
                Assert.AreEqual(1, pleaseCount, "1 occurence");

                // Get word enumerator.
                IDictionaryEnumerator enumerator = simpleIndex2.GetEnumerator();

                // Get words and their positions.
                while(enumerator.MoveNext())
                {
                    //current word
                    string word = (string) enumerator.Key;

                    // its positions
                    IList positions = (IList) enumerator.Value;
                }

                // save index data (to the db file).
                simpleIndex2.Save();
            }
            finally
            {
                if (indexSource != null)
                {
                    ((IDisposable) indexSource).Dispose();
                }
                if (indexSource2 != null)
                {
                    ((IDisposable) indexSource2).Dispose();
                }
            }
        }
    }
}

