/*
 * TestXmlDocumentIndexSource.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Globalization;
using System.IO;
using System.Text;
using NUnit.Framework;
using TopCoder.Document.Index.Persistence.Tests;
using TopCoder.Document.Index.WordStreams;

namespace TopCoder.Document.Index.Persistence.Xml.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="XmlDocumentIndexSource"/> class.
    /// </summary>
    [TestFixture]
    public class TestXmlDocumentIndexSource : TestCommon
    {
        /// <summary>
        /// The <see cref="XmlDocumentIndexSource"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private XmlDocumentIndexSource source;

        /// <summary>
        /// The document index xml file for testing.
        /// </summary>
        private const string docFile = @"..\..\test_files\Document0.xml";

        /// <summary>
        /// The original (unmodified) document index xml file.
        /// </summary>
        private const string docOrigFile = @"..\..\test_files\Document0.orig.xml";

        /// <summary>
        /// The assembly name for XmlDocumentIndexSource
        /// </summary>
        private const string assemblyName = "TopCoder.Document.Index.Persistence";

        /// <summary>
        /// The class name for XmlDocumentIndexSource.
        /// </summary>
        private const string className = "TopCoder.Document.Index.Persistence.Xml.XmlDocumentIndexSource";

        /// <summary>
        /// Temporary subfiles names.
        /// </summary>
        private const string subFiles = @"..\..\test_files\MultipleSub.Test (EraseMe).xml";

        /// <summary>
        /// Initializes the <see cref="XmlDocumentIndexSource"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // prepare sample doc index file
            TearDown();

            // prepare an XmlDocumentIndexSource for Document0.xml (10 words: 6 unique)
            source = new XmlDocumentIndexSource(docFile);
            string[] doc0 =
                "TopCoder Open 2005 Document Index Persistence TopCoder TopCoder TopCoder Open".Split(' ');
            int i = 0;
            foreach (string word in doc0)
            {
                // add splitted words
                source.Add(new WordInfo(word, i++));
            }
        }

        /// <summary>
        /// Cleans up by replacing the collection file with the unmodified one.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            File.Copy(docOrigFile, docFile, true);

            // erase any subfiles created
            string[] subs = Directory.GetFileSystemEntries(@"..\..\test_files", "*EraseMe*");
            foreach (string sub in subs)
            {
                File.Delete(sub);
            }
        }

        /// <summary>
        /// Tests constructor() accuracy and that FilePath is initially null
        /// and no words is stored (which is tested via GetEnumerator and indexer).
        /// </summary>
        [Test]
        public void TestCtor()
        {
            // prepare a blank XmlDocumentIndexSource
            source = new XmlDocumentIndexSource();
            Assert.AreEqual(null, source.FilePath, "Initially FilePath is null");

            // test via GetEnumerator
            IDictionaryEnumerator it = source.GetEnumerator();
            Assert.IsFalse(it.MoveNext(), "Nothing to enumerate");

            // test via this indexer
            Assert.IsNull(source["TopCoder"], "TopCoder not exists");
            Assert.IsNull(source["2005"], "2005 not exists");
        }

        /// <summary>
        /// Tests constructor(String) accuracy and that FilePath is initialized properly,
        /// and no index is stored (which is tested via GetEnumerator and indexer) initially.
        /// </summary>
        [Test]
        public void TestCtorString()
        {
            // prepare a XmlDocumentIndexSource to initialize a file
            source = new XmlDocumentIndexSource(docFile);
            Assert.AreEqual(docFile, source.FilePath, "FilePath is initialized");

            // test via GetEnumerator
            IDictionaryEnumerator it = source.GetEnumerator();
            Assert.IsFalse(it.MoveNext(), "Nothing to enumerate");

            // test via this indexer
            Assert.IsNull(source["TopCoder"], "TopCoder not exists");
            Assert.IsNull(source["2005"], "2005 not exists");
        }

        /// <summary>
        /// Tests constructor(String) given null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorStringNull()
        {
            new XmlDocumentIndexSource(null);
        }

        /// <summary>
        /// Tests constructor(String) given empty string argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringEmpty()
        {
            new XmlDocumentIndexSource(" ");
        }

        /// <summary>
        /// Tests constructor(String) given a path that points to a directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringDirectory()
        {
            new XmlDocumentIndexSource(@"..\..\test_files");
        }

        /// <summary>
        /// Tests constructor(String) given a path that points to the current directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringDirectoryCurrent()
        {
            new XmlDocumentIndexSource(".");
        }

        /// <summary>
        /// Tests constructor(String) given a path that points to the parent directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringDirectoryParent()
        {
            new XmlDocumentIndexSource("..");
        }

        /// <summary>
        /// Tests constructor(String) given a malformed path that contains a &gt; char.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringMalformedIllegalChar()
        {
            new XmlDocumentIndexSource(@"..\..\test_files\malformed>name");
        }

        /// <summary>
        /// Tests constructor(String) given a very long path (malformed).
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringMalformedLong()
        {
            // build a very long file name
            StringBuilder path = new StringBuilder();
            path.Append('a', 100000);
            new XmlDocumentIndexSource(path.ToString());
        }

        /// <summary>
        /// Tests SyncRoot accuracy, that it returns consistent object for locking.
        /// </summary>
        [Test]
        public void TestSyncRoot()
        {
            Assert.IsNotNull(source.SyncRoot, "A locking object is provided");
            Assert.AreSame(source.SyncRoot, source.SyncRoot,
                "Consistent object for locking is not returned");
        }

        /// <summary>
        /// Tests IndexInfo accuracy.
        /// </summary>
        [Test]
        public void TestIndexInfo()
        {
            IndexInfo idx = source.IndexInfo;
            Assert.IsTrue(idx.AssemblyName.StartsWith(assemblyName), "AssemblyName is incorrect");
            Assert.AreEqual(className, idx.ClassName, "ClassName is incorrect");
            Assert.AreEqual(docFile, ((SimpleIndexParameters) idx.Parameters).FileName,
                "Parameters.FileName is incorrect");
        }

        /// <summary>
        /// Tests IndexInfo when no file is given yet.
        /// Expecting InvalidOperationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestIndexInfoNoFilePath()
        {
            source = new XmlDocumentIndexSource();
            IndexInfo idx = source.IndexInfo;
        }

        /// <summary>
        /// Tests Culture getter and setter accuracy.
        /// </summary>
        [Test]
        public void TestCulture()
        {
            // Culture on default empty instance
            source = new XmlDocumentIndexSource();
            Assert.AreEqual("en-US", source.Culture.Name, "Default Culture is en-us");

            source.Culture = new CultureInfo("pl-PL");
            Assert.AreEqual("pl-PL", source.Culture.Name, "Culture changed");
        }

        /// <summary>
        /// Tests Culture setter given null reference.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCultureNull()
        {
            source.Culture = null;
        }

        /// <summary>
        /// Tests Culture setter when the words list is not empty.
        /// Expecting InvalidOperationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestCultureWordsNotEmpty()
        {
            source.Culture = new CultureInfo("ja-JP");
        }

        /// <summary>
        /// Tests CompareOptions getter and setter accuracy.
        /// </summary>
        [Test]
        public void TestCompareOptions()
        {
            // CompareOptions on default empty instance
            source = new XmlDocumentIndexSource();
            Assert.AreEqual(CompareOptions.None, source.CompareOptions, "Default CompareOptions is None");

            source.CompareOptions = CompareOptions.StringSort;
            Assert.AreEqual(CompareOptions.StringSort, source.CompareOptions, "CompareOptions changed");
        }

        /// <summary>
        /// Tests CompareOptions setter when the words list is not empty.
        /// Expecting InvalidOperationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestCompareOptionsWordsNotEmpty()
        {
            source.CompareOptions = CompareOptions.IgnoreWidth;
        }

        /// <summary>
        /// Tests FilePath setter and getter accuracy.
        /// </summary>
        [Test]
        public void TestFilePath()
        {
            string newPath = @"..\..\test_files\Document1.xml";
            Assert.AreEqual(docFile, source.FilePath, "FilePath is initialized incorrectly");
            source.FilePath = newPath;
            Assert.AreEqual(newPath, source.FilePath, "Setting FilePath to a new value");
        }

        /// <summary>
        /// Tests setting FilePath to a null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestFilePathNull()
        {
            source.FilePath = null;
        }

        /// <summary>
        /// Tests setting FilePath to an empty string argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathEmpty()
        {
            source.FilePath = " ";
        }

        /// <summary>
        /// Tests setting FilePath to a directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathDirectory()
        {
            source.FilePath = @"..\..\test_files";
        }

        /// <summary>
        /// Tests setting FilePath to the current directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathDirectoryCurrent()
        {
            source.FilePath = ".";
        }

        /// <summary>
        /// Tests setting FilePath to the parent directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathDirectoryParent()
        {
            source.FilePath = "..";
        }

        /// <summary>
        /// Tests setting FilePath to a malformed path that contains a &gt; char.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathMalformedIllegalChar()
        {
            source.FilePath = @"..\..\test_files\malformed>name";
        }

        /// <summary>
        /// Tests setting FilePath to a very long path (malformed).
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathMalformedLong()
        {
            // build a very long file name
            StringBuilder path = new StringBuilder();
            path.Append('a', 100000);
            source.FilePath = path.ToString();
        }

        /// <summary>
        /// Tests NumberOfSubFiles getter and setter accuracy.
        /// </summary>
        [Test]
        public void TestNumberOfSubFiles()
        {
            Assert.AreEqual(0, source.NumberOfSubFiles, "Default value is 0");

            // change to 2
            source.NumberOfSubFiles = 2;
            Assert.AreEqual(2, source.NumberOfSubFiles, "NumberOfSubFiles changed");
        }

        /// <summary>
        /// Tests NumberOfSubFiles given negative value.
        /// Expecting ArgumentOutOfRangeException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void TestNumberOfSubFilesNegative()
        {
            source.NumberOfSubFiles = -1;
        }

        /// <summary>
        /// Tests NumberOfSubFiles given value greater than the number of words.
        /// Expecting ArgumentOutOfRangeException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void TestNumberOfSubFilesGreaterThanWordsCount()
        {
            source.NumberOfSubFiles = 7;
        }

        /// <summary>
        /// Tests get-indexer accuracy as initialized in SetUp().
        /// </summary>
        [Test]
        public void TestIndexerGet()
        {
            // 2005
            IList pos = source["2005"];
            Assert.AreEqual(1, pos.Count, "2005 count");
            Assert.AreEqual(2, pos[0], "2005 pos");

            // Document
            pos = source["Document"];
            Assert.AreEqual(1, pos.Count, "Document count");
            Assert.AreEqual(3, pos[0], "Document pos");

            // Index
            pos = source["Index"];
            Assert.AreEqual(1, pos.Count, "Index count");
            Assert.AreEqual(4, pos[0], "Index pos");

            // Open
            pos = source["Open"];
            Assert.AreEqual(2, pos.Count, "Open count");
            Assert.AreEqual(1, pos[0], "Open 1 pos");
            Assert.AreEqual(9, pos[1], "Open 2 pos");

            // Persistence
            pos = source["Persistence"];
            Assert.AreEqual(1, pos.Count, "Persistence count");
            Assert.AreEqual(5, pos[0], "Persistence pos");

            // TopCoder
            pos = source["TopCoder"];
            Assert.AreEqual(4, pos.Count, "TopCoder count");
            Assert.AreEqual(0, pos[0], "TopCoder 1 pos");
            Assert.AreEqual(6, pos[1], "TopCoder 2 pos");
            Assert.AreEqual(7, pos[2], "TopCoder 3 pos");
            Assert.AreEqual(8, pos[3], "TopCoder 4 pos");

            Assert.IsNull(source["NotExists"], "Word not exists");
        }

        /// <summary>
        /// Tests get-indexer given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerGetNull()
        {
            IList pos = source[null];
        }

        /// <summary>
        /// Tests get-indexer given empty word.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerGetEmpty()
        {
            IList pos = source[" "];
        }

        /// <summary>
        /// Tests set-indexer accuracy by setting a new word.
        /// </summary>
        [Test]
        public void TestIndexerSet()
        {
            // create a new position list
            IList pos = new ArrayList();
            pos.Add(12345);
            pos.Add(678901234567L);
            pos.Add(0);
            source["new word !@<>"] = pos;

            IList assert = source["new word !@<>"];
            Assert.AreEqual(3, assert.Count, "Assert count");
            Assert.AreEqual(12345, assert[0], "Assert 1st element");
            Assert.AreEqual(678901234567L, assert[1], "Assert 2nd element");
            Assert.AreEqual(0, assert[2], "Assert 3rd element");
        }

        /// <summary>
        /// Tests set-indexer given null word.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerSetNullWord()
        {
            IList pos = new ArrayList();
            pos.Add(10);
            source[null] = pos;
        }

        /// <summary>
        /// Tests set-indexer given empty word.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerSetEmpty()
        {
            IList pos = new ArrayList();
            pos.Add(10);
            source[" "] = pos;
        }

        /// <summary>
        /// Tests set-indexer given null IList.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerSetNullIList()
        {
            source["testNull"] = null;
        }

        /// <summary>
        /// Tests set-indexer given existing word.
        /// It should overwrite existing entry.
        /// </summary>
        [Test]
        public void TestIndexerSetExist()
        {
            IList newPos = new ArrayList();
            newPos.Add(1789);
            newPos.Add(34567890123456);
            source["2005"] = newPos;

            IList assert = source["2005"];
            Assert.AreEqual(2, assert.Count, "Assert count");
            Assert.AreEqual(1789, assert[0], "Assert 1st element");
            Assert.AreEqual(34567890123456, assert[1], "Assert 2nd element");
        }

        /// <summary>
        /// Tests set-indexer given empty List.
        /// It should overwrite existing entry.
        /// </summary>
        [Test]
        public void TestIndexerSetEmptyList()
        {
            source["Open"] = new ArrayList();

            IList assert = source["Open"];
            Assert.AreEqual(0, assert.Count, "Assert count");
        }

        /// <summary>
        /// Tests set-indexer given non long/integer entry.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerNonLong()
        {
            IList pos = new ArrayList();
            pos.Add('a');
            source["open"] = pos;
        }

        /// <summary>
        /// Tests set-indexer given null entry in the IList.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerNullEntry()
        {
            IList pos = new ArrayList();
            pos.Add(null);
            source["null"] = pos;
        }

        /// <summary>
        /// Tests set-indexer given negative value.
        /// </summary>
        [Test]
        public void TestIndexerSetNegative()
        {
            // create a new position list
            IList pos = new ArrayList();
            pos.Add(-12345);
            pos.Add(-678901234567L);
            source["topcoder"] = pos;

            IList assert = source["topcoder"];
            Assert.AreEqual(2, assert.Count, "Assert count");
            Assert.AreEqual(-12345, assert[0], "Assert 1st element");
            Assert.AreEqual(-678901234567L, assert[1], "Assert 2nd element");
        }

        /// <summary>
        /// Tests indexer trims the word.
        /// </summary>
        [Test]
        public void TestIndexerTrim()
        {
            IList pos = new ArrayList();
            pos.Add(11);
            source[" Trim Test  "] = pos;

            IList assert = source["Trim Test"];
            Assert.AreEqual(1, assert.Count, "Spaces are trimmed");
            Assert.AreEqual(11, assert[0], "Pos");
        }

        /// <summary>
        /// Tests Add accuracy by adding a new position to an existing word.
        /// </summary>
        [Test]
        public void TestAdd()
        {
            // add a new position for existing word
            source.Add(new WordInfo("Index", 10));

            IList assert = source["Index"];
            Assert.AreEqual(2, assert.Count, "Adds a new entry");
            Assert.AreEqual(4, assert[0], "Old element");
            Assert.AreEqual(10, assert[1], "Newly added element");
        }

        /// <summary>
        /// Tests Add accuracy by adding a new position to a non-existing word (case-sensitive).
        /// </summary>
        [Test]
        public void TestAddNewWord()
        {
            // add a new word
            source.Add(new WordInfo("index", 10));

            IList assert = source["index"];
            Assert.AreEqual(1, assert.Count, "Adds a new entry");
            Assert.AreEqual(10, assert[0], "New position");

            Assert.AreEqual(1, source["Index"].Count, "Old element still exists");
            Assert.AreEqual(4, source["Index"][0], "Old element value");
        }

        /// <summary>
        /// Tests Add accuracy by adding IList containing duplicate positions.
        /// Nothing should be removed.
        /// </summary>
        [Test]
        public void TestAddDuplicate()
        {
            // add a duplicate position for existing word
            source.Add(new WordInfo("Index", 4));

            IList assert = source["Index"];
            Assert.AreEqual(2, assert.Count, "Adds a new entry");
            Assert.AreEqual(4, assert[0], "Old element");
            Assert.AreEqual(4, assert[1], "Newly added element");
        }

        /// <summary>
        /// Tests Add trim the word.
        /// </summary>
        [Test]
        public void TestAddTrim()
        {
            source.Add(new WordInfo(" Trim Test  ", 11));

            IList assert = source["Trim Test"];
            Assert.AreEqual(1, assert.Count, "Spaces are trimmed");
            Assert.AreEqual(11, assert[0], "Pos");
        }

        /// <summary>
        /// Tests Add given null word.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddNullName()
        {
            source.Add(null);
        }

        /// <summary>
        /// Tests Add given empty word.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddEmpty()
        {
            source.Add(new WordInfo(" ", 1));
        }

        /// <summary>
        /// Tests Remove accuracy by removing existing name.
        /// </summary>
        [Test]
        public void TestRemove()
        {
            Assert.IsNotNull(source["TopCoder"], "TopCoder before removed");
            source.Remove("TopCoder");
            Assert.IsNull(source["TopCoder"], "TopCoder is removed");
        }

        /// <summary>
        /// Tests Remove by removing non-existing name (name is case-sensitive).
        /// Nothing should be removed.
        /// </summary>
        [Test]
        public void TestRemoveNotExist()
        {
            Assert.IsNotNull(source["TopCoder"], "TopCoder exists");
            Assert.IsNotNull(source["Open"], "Open exists");
            source.Remove("topcoder");
            Assert.IsNotNull(source["TopCoder"], "TopCoder still exists");
            Assert.IsNotNull(source["Open"], "Open still exists");
        }

        /// <summary>
        /// Tests Remove given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestRemoveNull()
        {
            source.Remove(null);
        }

        /// <summary>
        /// Tests Remove given an empty string index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestRemoveEmpty()
        {
            source.Remove(" ");
        }

        /// <summary>
        /// Tests GetEnumerator accuracy by iterating existing collection.
        /// </summary>
        [Test]
        public void TestGetEnumerator()
        {
            IDictionaryEnumerator it = source.GetEnumerator();

            // 2005
            Assert.IsTrue(it.MoveNext(), "1st entry");
            IList pos = (IList) it.Value;
            Assert.AreEqual("2005", it.Key, "2005");
            Assert.AreEqual(1, pos.Count, "2005 count");
            Assert.AreEqual(2, pos[0], "2005 pos");

            // Document
            Assert.IsTrue(it.MoveNext(), "2nd entry");
            pos = (IList) it.Value;
            Assert.AreEqual("Document", it.Key, "Document");
            Assert.AreEqual(1, pos.Count, "Document count");
            Assert.AreEqual(3, pos[0], "Document pos");

            // Index
            Assert.IsTrue(it.MoveNext(), "3rd entry");
            pos = (IList) it.Value;
            Assert.AreEqual("Index", it.Key, "Index");
            Assert.AreEqual(1, pos.Count, "Index count");
            Assert.AreEqual(4, pos[0], "Index pos");

            // Open
            Assert.IsTrue(it.MoveNext(), "4th entry");
            pos = (IList) it.Value;
            Assert.AreEqual("Open", it.Key, "Open");
            Assert.AreEqual(2, pos.Count, "Open count");
            Assert.AreEqual(1, pos[0], "Open 1 pos");
            Assert.AreEqual(9, pos[1], "Open 2 pos");

            // Persistence
            Assert.IsTrue(it.MoveNext(), "5th entry");
            pos = (IList) it.Value;
            Assert.AreEqual("Persistence", it.Key, "Persistence");
            Assert.AreEqual(1, pos.Count, "Persistence count");
            Assert.AreEqual(5, pos[0], "Persistence pos");

            // TopCoder
            Assert.IsTrue(it.MoveNext(), "6th entry");
            pos = (IList) it.Value;
            Assert.AreEqual("TopCoder", it.Key, "TopCoder");
            Assert.AreEqual(4, pos.Count, "TopCoder count");
            Assert.AreEqual(0, pos[0], "TopCoder 1 pos");
            Assert.AreEqual(6, pos[1], "TopCoder 2 pos");
            Assert.AreEqual(7, pos[2], "TopCoder 3 pos");
            Assert.AreEqual(8, pos[3], "TopCoder 4 pos");

            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Add and GetEnumerator accuracy that a new element is added.
        /// </summary>
        [Test]
        public void TestGetEnumeratorAdd()
        {
            source.Add(new WordInfo("new Word", 10));
            IDictionaryEnumerator it = source.GetEnumerator();

            // we have 7 entries now
            for (int i = 0; i < 7; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Entry: " + i);
            }
            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Remove and GetEnumerator accuracy that an element is removed.
        /// </summary>
        [Test]
        public void TestGetEnumeratorRemove()
        {
            source.Remove("Index");
            IDictionaryEnumerator it = source.GetEnumerator();

            // we have 5 entries now
            for (int i = 0; i < 5; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Entry: " + i);
            }
            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Contains accuracy by checking existing and non-existing name.
        /// </summary>
        [Test]
        public void TestContains()
        {
            Assert.IsTrue(source.Contains("2005"), "1st entry");
            Assert.IsTrue(source.Contains("Document"), "2nd entry");
            Assert.IsTrue(source.Contains("Index"), "3rd entry");
            Assert.IsTrue(source.Contains("Open"), "4th entry");
            Assert.IsTrue(source.Contains("Persistence"), "5th entry");
            Assert.IsTrue(source.Contains("TopCoder"), "6th entry");
            Assert.IsFalse(source.Contains("document"), "Name is case-sensitive");
        }

        /// <summary>
        /// Tests Contains given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestContainsNull()
        {
            source.Contains(null);
        }

        /// <summary>
        /// Tests Contains given an empty string index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestContainsEmpty()
        {
            source.Contains(" ");
        }

        /// <summary>
        /// Tests Contains given a word with empty position.
        /// False is expected.
        /// </summary>
        [Test]
        public void TestContainsEmptyPosition()
        {
            source["NoPos"] = new ArrayList();
            Assert.IsFalse(source.Contains("NoPos"), "The array is empty");
        }

        /// <summary>
        /// Tests Clear removes all collections.
        /// Assert that it is cleared via Contains, indexer, and GetEnumerator.
        /// </summary>
        [Test]
        public void TestClear()
        {
            source.Clear();
            Assert.IsFalse(source.Contains("2005"), "1st entry");
            Assert.IsFalse(source.Contains("Document"), "2nd entry");
            Assert.IsFalse(source.Contains("Index"), "3rd entry");
            Assert.IsFalse(source.Contains("Open"), "4th entry");
            Assert.IsFalse(source.Contains("Persistence"), "5th entry");
            Assert.IsFalse(source.Contains("TopCoder"), "6th entry");

            Assert.IsFalse(source.GetEnumerator().MoveNext(), "No entry in enumerator");
        }

        /// <summary>
        /// Tests Compare for default Culture and CompareOptions.
        /// </summary>
        [Test]
        public void TestCompare()
        {
            Assert.AreEqual(0, source.Compare("AC", "AC"), "Equals");
            Assert.AreEqual(1, source.Compare("AC", "ac"), "Lowercase appears first");
            Assert.AreEqual(-1, source.Compare("13", "AC"), "Digits appears first");
        }

        /// <summary>
        /// Tests Compare given non-string first argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCompareNonString1()
        {
            source.Compare(10, "10");
        }

        /// <summary>
        /// Tests Compare given non-string second argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCompareNonString2()
        {
            source.Compare("10", 10);
        }

        /// <summary>
        /// Tests Compare with invariant culture.
        /// </summary>
        [Test]
        public void TestCompareInvariantCulture()
        {
            source.Clear();
            source.Culture = CultureInfo.InvariantCulture;
            Assert.AreEqual(0, source.Compare("AC", "AC"), "Equals");
            Assert.AreEqual(1, source.Compare("AC", "ac"), "Lowercase appears first");
            Assert.AreEqual(-1, source.Compare("13", "AC"), "Digits appears first");
        }

        /// <summary>
        /// Tests Compare with Ordinal CompareOptions.
        /// </summary>
        [Test]
        public void TestCompareOrdinal()
        {
            source.Clear();
            source.CompareOptions = CompareOptions.Ordinal;
            Assert.AreEqual(0, source.Compare("AC", "AC"), "Equals");
            Assert.IsTrue(source.Compare("AC", "ac") < 0, "Lowercase appears last");
            Assert.IsTrue(source.Compare("13", "AC") < 0, "Digits appears first");
        }

        /// <summary>
        /// Tests Compare with case-insensitive CompareOptions.
        /// </summary>
        [Test]
        public void TestCompareCaseInsensitive()
        {
            source.Clear();
            source.CompareOptions = CompareOptions.IgnoreCase;
            Assert.AreEqual(0, source.Compare("AC", "ac"), "Equals");
            Assert.AreEqual(-1, source.Compare("AC", "af"), "AC appears first (case-insensitive)");
            Assert.AreEqual(-1, source.Compare("13", "AC"), "Digits appears first");
        }

        /// <summary>
        /// Tests case-insensitive CompareOptions with Add/Remove/Contains and indexer.
        /// </summary>
        [Test]
        public void TestCaseInsensitive()
        {
            source.Clear();
            source.CompareOptions = CompareOptions.IgnoreCase;

            // ABC: 10
            IList pos0 = new ArrayList();
            pos0.Add(10);
            source["ABC"] = pos0;

            // abcd: 20
            IList pos1 = new ArrayList();
            pos1.Add(20);
            source["abcd"] = pos1;

            // aBc: 30 (this should overwrite ABC)
            IList pos2 = new ArrayList();
            pos2.Add(30);
            source["aBc"] = pos2;

            // test with get-indexer
            Assert.AreEqual(30, source["abc"][0], "Indexer getter abc is case-insensitive");
            Assert.AreEqual(20, source["AbCd"][0], "Indexer getter abcd is case-insensitive");

            // test with Add (2 positions registered now)
            source.Add(new WordInfo("AbC", 40));
            Assert.AreEqual(30, source["abc"][0], "Add0 should be case-insensitive");
            Assert.AreEqual(40, source["abc"][1], "Add1 should be case-insensitive");

            // test with Remove and Contains
            source.Remove("abc");
            Assert.IsNull(source["abc"], "Remove should be case-insensitive");
            Assert.IsFalse(source.Contains("aBC"), "Contains should be case-insensitive");

            // test Enumertor
            IDictionaryEnumerator it = source.GetEnumerator();
            Assert.IsTrue(it.MoveNext(), "abcd retrieved");
            Assert.AreEqual("abcd", it.Key, "abcd key");

            Assert.IsFalse(it.MoveNext(), "No more element");
        }

        /// <summary>
        /// Tests Load accuracy by loading an existing file (single).
        /// </summary>
        [Test]
        public void TestLoad()
        {
            // load sample document file
            source.Load(new SimpleIndexParameters(docFile));

            // 6 words should be loaded
            IDictionaryEnumerator it = source.GetEnumerator();
            for (int i = 0; i < 6; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Data is loaded: " + i);
            }
            Assert.IsFalse(it.MoveNext(), "No more data");

            // 2005
            IList pos = source["2005"];
            Assert.AreEqual(1, pos.Count, "2005 count");
            Assert.AreEqual(2, pos[0], "2005 pos");

            // Document
            pos = source["Document"];
            Assert.AreEqual(1, pos.Count, "Document count");
            Assert.AreEqual(3, pos[0], "Document pos");

            // Index
            pos = source["Index"];
            Assert.AreEqual(1, pos.Count, "Index count");
            Assert.AreEqual(4, pos[0], "Index pos");

            // Open
            pos = source["Open"];
            Assert.AreEqual(2, pos.Count, "Open count");
            Assert.AreEqual(1, pos[0], "Open 1 pos");
            Assert.AreEqual(9, pos[1], "Open 2 pos");

            // Persistence
            pos = source["Persistence"];
            Assert.AreEqual(1, pos.Count, "Persistence count");
            Assert.AreEqual(5, pos[0], "Persistence pos");

            // TopCoder
            pos = source["TopCoder"];
            Assert.AreEqual(4, pos.Count, "TopCoder count");
            Assert.AreEqual(0, pos[0], "TopCoder 1 pos");
            Assert.AreEqual(6, pos[1], "TopCoder 2 pos");
            Assert.AreEqual(7, pos[2], "TopCoder 3 pos");
            Assert.AreEqual(8, pos[3], "TopCoder 4 pos");
        }

        /// <summary>
        /// Tests Load accuracy by loading an existing files (multiple).
        /// </summary>
        [Test]
        public void TestLoadMultiple()
        {
            source.Load(new SimpleIndexParameters(@"..\..\test_files\Document2.xml"));

            // assert FilePath is reinitialized
            Assert.AreEqual(@"..\..\test_files\Document2.xml", source.FilePath, "FilePath is reinitialized");

            // 7 words should be loaded
            IDictionaryEnumerator it = source.GetEnumerator();
            for (int i = 0; i < 7; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Data is loaded: " + i);
            }
            Assert.IsFalse(it.MoveNext(), "No more data");

            // 2005
            IList pos = source["2005"];
            Assert.AreEqual(1, pos.Count, "2005 count");
            Assert.AreEqual(2, pos[0], "2005 pos");

            // Document
            pos = source["Document"];
            Assert.AreEqual(1, pos.Count, "Document count");
            Assert.AreEqual(3, pos[0], "Document pos");

            // Index
            pos = source["Index"];
            Assert.AreEqual(1, pos.Count, "Index count");
            Assert.AreEqual(4, pos[0], "Index pos");

            // Open
            pos = source["Open"];
            Assert.AreEqual(2, pos.Count, "Open count");
            Assert.AreEqual(1, pos[0], "Open 1 pos");
            Assert.AreEqual(9, pos[1], "Open 2 pos");

            // Persistence
            pos = source["Persistence"];
            Assert.AreEqual(1, pos.Count, "Persistence count");
            Assert.AreEqual(5, pos[0], "Persistence pos");

            // tcs
            pos = source["tcs"];
            Assert.AreEqual(1, pos.Count, "tcs count");
            Assert.AreEqual(10, pos[0], "tcs pos");

            // TopCoder
            pos = source["TopCoder"];
            Assert.AreEqual(4, pos.Count, "TopCoder count");
            Assert.AreEqual(0, pos[0], "TopCoder 1 pos");
            Assert.AreEqual(6, pos[1], "TopCoder 2 pos");
            Assert.AreEqual(7, pos[2], "TopCoder 3 pos");
            Assert.AreEqual(8, pos[3], "TopCoder 4 pos");
        }

        /// <summary>
        /// Tests Load given a file with case-insensitive words settings.
        /// </summary>
        [Test]
        public void TestLoadCompareOptionsCaseInsensitive()
        {
            // both word are considered equal and hence merged (empty spaces are always trimmed)
            base.WriteToFile("<Document><CompareOptions> IgnoreCase </CompareOptions><Culture> sk-SK </Culture>"
                + "<Words><WordsList>"
                + "<Word> ab </Word><Locations><Location>1</Location></Locations>"
                + "<Word>AB</Word><Locations><Location> 5 </Location></Locations>"
                + "</WordsList></Words></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));

            // assert two positions are loaded
            IList pos = source["Ab"];
            Assert.AreEqual(2, pos.Count, "2 positions");
            Assert.AreEqual(1, pos[0], "Position 0");
            Assert.AreEqual(5, pos[1], "Position 1");
        }

        /// <summary>
        /// Tests Load given null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestLoadNull()
        {
            source.Load(null);
        }

        /// <summary>
        /// Tests Load given Parameter which is not SimpleIndexParameters.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadNotSimpleIndexParameters()
        {
            source.Load(new MyParameters());
        }

        /// <summary>
        /// Tests Load given Parameter that points to a directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadDirectory()
        {
            source.Load(new SimpleIndexParameters(@"..\..\test_files"));
        }

        /// <summary>
        /// Tests Load given Parameter that points to the current directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadDirectoryCurrent()
        {
            source.Load(new SimpleIndexParameters("."));
        }

        /// <summary>
        /// Tests Load given Parameter that points to the parent directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadDirectoryParent()
        {
            source.Load(new SimpleIndexParameters(".."));
        }

        /// <summary>
        /// Tests Load given Parameter that points to a malformed path that contains a &gt; char.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadMalformedIllegalChar()
        {
            source.Load(new SimpleIndexParameters(@"..\..\test_files\malformed>name"));
        }

        /// <summary>
        /// Tests Load given Parameter that points to a very long path (malformed).
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadMalformedLong()
        {
            // build a very long file name
            StringBuilder path = new StringBuilder();
            path.Append('a', 100000);
            source.Load(new SimpleIndexParameters(path.ToString()));
        }

        /// <summary>
        /// Tests Load given Parameter that points to a non-existing file.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadNotExist()
        {
            source.Load(new SimpleIndexParameters(@"..\..\test_files\NotExists.xml"));
        }

        /// <summary>
        /// Tests Load given an empty file (missing root element).
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadEmptyFile()
        {
            base.WriteToFile("", docFile);
            source.Load(new SimpleIndexParameters(docFile));
        }

        /// <summary>
        /// Tests Load given a file with invalid root element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadInvalidRoot()
        {
            base.WriteToFile("<Doc/>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
        }

        /// <summary>
        /// Tests Load given a file with invalid Document element (no CompareOptions).
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadInvalidDocumentCompareOptions()
        {
            base.WriteToFile("<Document><Culture/><Words/></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
        }

        /// <summary>
        /// Tests Load given a file with invalid Document element (no Culture).
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadInvalidDocumentCulture()
        {
            base.WriteToFile("<Document><CompareOptions/><Words/></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
        }

        /// <summary>
        /// Tests Load given a file with empty CompareOptions element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadEmptyCompareOptions()
        {
            base.WriteToFile(
                "<Document><CompareOptions/><Culture>en-us</Culture><Words/></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
        }

        /// <summary>
        /// Tests Load given a file with empty Culture element.
        /// Expecting InvariantCulture is assigned.
        /// </summary>
        [Test]
        public void TestLoadEmptyCulture()
        {
            base.WriteToFile(
                "<Document><CompareOptions>None</CompareOptions><Culture/><Words/></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
            Assert.AreEqual(CultureInfo.InvariantCulture.DisplayName, source.Culture.DisplayName,
                "Invariant culture is used");
        }

        /// <summary>
        /// Tests Load given a file with unknown CompareOptions element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadUnknownCompareOptions()
        {
            base.WriteToFile("<Document><CompareOptions>x</CompareOptions>"
                + "<Culture>en-AU</Culture><Words/></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
            Assert.AreEqual(CultureInfo.InvariantCulture.DisplayName, source.Culture.DisplayName,
                "Invariant culture is used");
        }

        /// <summary>
        /// Tests Load given a file with unknown Culture element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadUnknownCulture()
        {
            base.WriteToFile("<Document><CompareOptions>None</CompareOptions>"
                + "<Culture>x</Culture><Words/></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
            Assert.AreEqual(CultureInfo.InvariantCulture.DisplayName, source.Culture.DisplayName,
                "Invariant culture is used");
        }

        /// <summary>
        /// Tests Load given a file with empty Word element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadEmptyWord()
        {
            base.WriteToFile("<Document><CompareOptions>None</CompareOptions><Culture>sk-SK</Culture>"
                + "<Words><WordsList>"
                + "<Word> </Word><Locations><Location>1</Location></Locations>"
                + "</WordsList></Words></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
        }

        /// <summary>
        /// Tests Load given a file with empty Location element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadEmptyLocation()
        {
            base.WriteToFile("<Document><CompareOptions>None</CompareOptions><Culture>sk-SK</Culture>"
                + "<Words><WordsList>"
                + "<Word>x</Word><Locations><Location> </Location></Locations>"
                + "</WordsList></Words></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
        }

        /// <summary>
        /// Tests Load given a file with non-integer Location element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadNonIntegerLocation()
        {
            base.WriteToFile("<Document><CompareOptions>None</CompareOptions><Culture>sk-SK</Culture>"
                + "<Words><WordsList>"
                + "<Word>x</Word><Locations><Location>y</Location></Locations>"
                + "</WordsList></Words></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
        }

        /// <summary>
        /// Tests Load preserves the existing data if any exception occurs (transactionality test).
        /// </summary>
        [Test]
        public void TestLoadTransact()
        {
            // load an invalid file (cause exception)
            try
            {
                base.WriteToFile("<Document/><Document>", docFile);
                source.Load(new SimpleIndexParameters(docFile));
                Assert.Fail("LoadIndexDataException expected");
            }
            catch (LoadIndexDataException)
            {
                // ok
            }

            // make sure existing data is preserved
            Assert.IsTrue(source.Contains("2005"), "1st entry");
            Assert.IsTrue(source.Contains("Document"), "2nd entry");
            Assert.IsTrue(source.Contains("Index"), "3rd entry");
            Assert.IsTrue(source.Contains("Open"), "4th entry");
            Assert.IsTrue(source.Contains("Persistence"), "5th entry");
            Assert.IsTrue(source.Contains("TopCoder"), "6th entry");
            Assert.IsFalse(source.Contains("document"), "Name is case-sensitive");
        }

        /// <summary>
        /// Tests Load given a file containing an empty words.
        /// </summary>
        [Test]
        public void TestLoadEmpty()
        {
            base.WriteToFile("<Document><CompareOptions>  IgnoreWidth </CompareOptions><Culture> uk-UA</Culture>"
                + "<Words/></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
            Assert.IsFalse(source.GetEnumerator().MoveNext(), "No data is loaded");
            Assert.AreEqual(CompareOptions.IgnoreWidth, source.CompareOptions, "CompareOptions");
            Assert.AreEqual("uk-UA", source.Culture.Name, "Culture");
        }

        /// <summary>
        /// Tests Load given a file containing &lt; &gt; chars in the name.
        /// </summary>
        [Test]
        public void TestLoadEscapedChars()
        {
            base.WriteToFile("<Document><CompareOptions>None</CompareOptions><Culture>en-US</Culture>"
                + "<Words><WordsList>"
                + "<Word> &lt;word &gt; </Word><Locations><Location>1</Location></Locations>"
                + "</WordsList></Words></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
            Assert.AreEqual(1, source["<word >"][0], "Escaped char");
        }

        /// <summary>
        /// Tests Load given a file containing CDATA tag in the name/filepath.
        /// </summary>
        [Test]
        public void TestLoadCData()
        {
            base.WriteToFile("<Document><CompareOptions>None</CompareOptions><Culture>en-US</Culture>"
                + "<Words><WordsList>"
                + "<Word><![CDATA[ <word&gt; ]]></Word><Locations><Location>5</Location></Locations>"
                + "</WordsList></Words></Document>", docFile);
            source.Load(new SimpleIndexParameters(docFile));
            Assert.AreEqual(5, source["<word&gt;"][0], "CData");
        }

        /// <summary>
        /// Tests Load resets the FilePath.
        /// </summary>
        [Test]
        public void TestLoadResetFilePath()
        {
            // load should reassign the FilePath
            // so we change FilePath temporarily to make sure it'd be reset
            string newPath = @"..\..\test_files\Document2.xml";
            source.FilePath = newPath;
            Assert.AreEqual(newPath, source.FilePath, "The FilePath is changed");
            source.Load(new SimpleIndexParameters(docFile));
            Assert.AreEqual(docFile, source.FilePath, "The FilePath is reset");
        }

        /// <summary>
        /// Tests Save accuracy by saving the XmlDocumentIndexSource initialized in SetUp().
        /// </summary>
        [Test]
        public void TestSave()
        {
            // the resulting file should be similar to Document0.xml
            string expected = base.ReadContent(docFile, true);

            // delete and save to new file
            File.Delete(docFile);
            source.Save();

            // re-read and assert
            string actual = base.ReadContent(docFile, true);
            Assert.AreEqual(expected, actual, "Written xml is incorrect");


            // reloading the file should not raise exception
            source.Load(new SimpleIndexParameters(source.FilePath));
        }

        /// <summary>
        /// Tests Save throws InvalidOperationException
        /// if no FilePath is specified yet.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestSaveNull()
        {
            // blank FilePath initially
            source = new XmlDocumentIndexSource();
            source.Save();
        }

        /// <summary>
        /// Tests Save given empty document.
        /// </summary>
        [Test]
        public void TestSaveEmpty()
        {
            source.Clear();
            source.Save();
            string actual = base.ReadContent(docFile, true);
            Assert.AreEqual(@"<?xml version=""1.0"" encoding=""utf-8""?>"
                + "<Document><CompareOptions>None</CompareOptions><Culture>en-US</Culture>"
                + "<Words><WordsList /></Words></Document>", actual, "Written xml is incorrect");

            // reloading the file should not raise exception
            source.Load(new SimpleIndexParameters(source.FilePath));
        }

        /// <summary>
        /// Tests Save given collection with characters that must be escaped.
        /// </summary>
        [Test]
        public void TestSaveEscapedChars()
        {
            source.Clear();
            source.Add(new WordInfo("<Test />", 123));
            source.Save();
            string actual = base.ReadContent(docFile, true);
            Assert.AreEqual(@"<?xml version=""1.0"" encoding=""utf-8""?>"
                + "<Document><CompareOptions>None</CompareOptions><Culture>en-US</Culture>"
                + "<Words><WordsList>"
                + "<Word>&lt;Test /&gt;</Word><Locations><Location>123</Location></Locations>"
                + "</WordsList></Words></Document>", actual, "Written xml is incorrect");

            // reloading the file should not raise exception
            source.Load(new SimpleIndexParameters(source.FilePath));
        }

        /// <summary>
        /// Tests Save given a word containing an empty position list.
        /// Expecting that the word is not written to the file.
        /// </summary>
        [Test]
        public void TestSaveEmptyPositionList()
        {
            source.Clear();
            source["Empty"] = new ArrayList();
            Assert.IsTrue(source.GetEnumerator().MoveNext(), "An empty position data exists");

            source.Save();
            string actual = base.ReadContent(docFile, true);
            Assert.AreEqual(@"<?xml version=""1.0"" encoding=""utf-8""?>"
                + "<Document><CompareOptions>None</CompareOptions><Culture>en-US</Culture>"
                + "<Words><WordsList />"
                + "</Words></Document>", actual, "Written xml is incorrect");

            // reloading the file should not raise exception
            source.Load(new SimpleIndexParameters(source.FilePath));
            Assert.IsFalse(source.GetEnumerator().MoveNext(), "No data");
        }

        /// <summary>
        /// Tests Save given a word containing duplicate position.
        /// Expecting that the duplicate position is not written to the file.
        /// </summary>
        [Test]
        public void TestSaveDuplicatePositionList()
        {
            source.Clear();
            IList pos = new ArrayList();
            pos.Add(123);
            pos.Add(123);
            pos.Add(456);
            source["Dup"] = pos;
            source.Save();
            string actual = base.ReadContent(docFile, true);
            Assert.AreEqual(@"<?xml version=""1.0"" encoding=""utf-8""?>"
                + "<Document><CompareOptions>None</CompareOptions><Culture>en-US</Culture>"
                + "<Words><WordsList>"
                + "<Word>Dup</Word><Locations><Location>123</Location><Location>456</Location></Locations>"
                + "</WordsList></Words></Document>", actual, "Written xml is incorrect");

            // reloading the file should not raise exception
            source.Load(new SimpleIndexParameters(source.FilePath));
        }

        /// <summary>
        /// Tests Save and Load.
        /// </summary>
        [Test]
        public void TestSaveAndLoad()
        {
            // create 2 indices and save
            source.Clear();
            source.Add(new WordInfo("<Doc1/>", 123));
            source.Add(new WordInfo(" Doc2", 1234567890));
            source.Add(new WordInfo(" Doc2 ", 98765));
            source.Save();

            // clear and reload
            source.Clear();
            Assert.IsFalse(source.GetEnumerator().MoveNext(), "Data cleared");
            source.Load(new SimpleIndexParameters(docFile));

            // assert Doc1
            IList pos = source["<Doc1/>"];
            Assert.AreEqual(1, pos.Count, "Doc1 has 1 position");
            Assert.AreEqual(123, pos[0], "Doc1 pos");

            // assert Doc2
            pos = source["Doc2"];
            Assert.AreEqual(2, pos.Count, "Doc2 has 2 positions");
            Assert.AreEqual(1234567890, pos[0], "Doc2-1 pos");
            Assert.AreEqual(98765, pos[1], "Doc2-2 pos");
        }

        /// <summary>
        /// Tests Save and Load with multiple sub-files.
        /// Asserts and compares it with Document2.xml.
        /// </summary>
        [Test]
        public void TestSaveAndLoadMultiple()
        {
            // make it 7 words (add tcs)
            source.Add(new WordInfo("tcs", 10));

            // split into 2 files
            source.FilePath = subFiles;
            source.NumberOfSubFiles = 2;
            source.Save();

            // the result should be similar to Document2*.xml
            string[] files = Directory.GetFileSystemEntries(@"..\..\test_files", "*EraseMe*.xml");
            Assert.AreEqual(3, files.Length, "3 files are created");

            // assert subfiles
            string masterFile = base.ReadContent(subFiles, true);
            string[] subs = base.XPathSelect(masterFile, "/Document/Words/Files/FilePath");

            // assert 1st sub file
            string actual = base.ReadContent(subs[0], true);
            string expected = base.ReadContent(
                @"..\..\test_files\Document2(883bba12-afad-42f7-9780-86359f285cad).xml", true);
            Assert.AreEqual(expected, actual, "1st sub file is different");

            // assert 2nd sub file
            actual = base.ReadContent(subs[1], true);
            expected = base.ReadContent(
                @"..\..\test_files\Document2(b3aed422-4354-46ad-b88d-d0f07285054b).xml", true);
            Assert.AreEqual(expected, actual, "2nd sub file is different");

            // reload the sub files (no exception is expected)
            source.Load(new SimpleIndexParameters(subFiles));

            // 7 unique words
            IDictionaryEnumerator it = source.GetEnumerator();
            for (int i = 0; i < 7; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Entry: " + i);
            }
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Save and Load with multiple sub-files.
        /// Given 7 unique words and 4 sub-files, the first 3 files should contain 2 entries,
        /// the last one contains 1 entry.
        /// </summary>
        [Test]
        public void TestSaveAndLoadMultipleDivide()
        {
            // split into 4 files (2 + 2 + 2 + 1)
            source.FilePath = subFiles;
            source.NumberOfSubFiles = 4;

            // make it 7 words (add tcs)
            source.Add(new WordInfo("tcs", 10));
            source.Save();

            // 4 sub files + 1 main file
            string[] files = Directory.GetFileSystemEntries(@"..\..\test_files", "*EraseMe*.xml");
            Assert.AreEqual(5, files.Length, "5 files are created");

            // assert subfiles
            string masterFile = base.ReadContent(subFiles, true);
            string[] subs = base.XPathSelect(masterFile, "/Document/Words/Files/FilePath");
            Assert.AreEqual(4, subs.Length, "4 sub files registered");

            // assert 1st sub file (2 entries)
            string actual = base.ReadContent(subs[0], true);
            string[] words = base.XPathSelect(actual, "/WordsList/Word");
            Assert.AreEqual(2, words.Length, "2 words in 1st subfile");
            Assert.AreEqual("2005", words[0], "1st word in 1st subfile");
            Assert.AreEqual("Document", words[1], "2nd word in 1st subfile");

            // assert 2nd sub file (2 entries)
            actual = base.ReadContent(subs[1], true);
            words = base.XPathSelect(actual, "/WordsList/Word");
            Assert.AreEqual(2, words.Length, "2 words in 2nd subfile");
            Assert.AreEqual("Index", words[0], "1st word in 2nd subfile");
            Assert.AreEqual("Open", words[1], "2nd word in 2nd subfile");

            // assert 3rd sub file (2 entries)
            actual = base.ReadContent(subs[2], true);
            words = base.XPathSelect(actual, "/WordsList/Word");
            Assert.AreEqual(2, words.Length, "2 words in 3rd subfile");
            Assert.AreEqual("Persistence", words[0], "1st word in 3rd subfile");
            Assert.AreEqual("tcs", words[1], "2nd word in 3rd subfile");

            // assert 4th sub file (1 entry)
            actual = base.ReadContent(subs[3], true);
            words = base.XPathSelect(actual, "/WordsList/Word");
            Assert.AreEqual(1, words.Length, "1 word in 4th subfile");
            Assert.AreEqual("TopCoder", words[0], "1st word in 4th subfile");

            // reload the sub files (no exception is expected)
            source.Load(new SimpleIndexParameters(subFiles));

            // 7 unique words
            IDictionaryEnumerator it = source.GetEnumerator();
            for (int i = 0; i < 7; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Entry: " + i);
            }
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Save and Load given NumberOfSubFiles greater than word.
        /// Since it's not allowed to set NumberOfSubFiles greater than word count,
        /// we could set it first, then clear the words.
        /// </summary>
        [Test]
        public void TestSaveAndLoadMultipleNoOfSubFilesTooBig()
        {
            // split into 3 files
            source.FilePath = subFiles;
            source.NumberOfSubFiles = 3;

            // make it just one word with 2 positions
            source.Clear();
            source.Add(new WordInfo("tcs", 10));
            source.Add(new WordInfo("tcs", 11));
            source.Save();

            // 1 sub file + 1 main file only is created
            string[] files = Directory.GetFileSystemEntries(@"..\..\test_files", "*EraseMe*.xml");
            Assert.AreEqual(2, files.Length, "2 files are created");

            // assert subfiles
            string masterFile = base.ReadContent(subFiles, true);
            string[] subs = base.XPathSelect(masterFile, "/Document/Words/Files/FilePath");
            Assert.AreEqual(1, subs.Length, "1 sub file registered");

            // assert 1st sub file (1 entries)
            string actual = base.ReadContent(subs[0], true);
            string[] words = base.XPathSelect(actual, "/WordsList/Word");
            Assert.AreEqual(1, words.Length, "1 word in 1st subfile");
            Assert.AreEqual("tcs", words[0], "1st word in 1st subfile");
            string[] pos = base.XPathSelect(actual, "/WordsList/Locations/Location");
            Assert.AreEqual(2, pos.Length, "2 positions for the word");
            Assert.AreEqual("10", pos[0], "1st position");
            Assert.AreEqual("11", pos[1], "2nd position");

            // reload the sub files (no exception is expected)
            source.Load(new SimpleIndexParameters(subFiles));

            // 1 unique word
            IDictionaryEnumerator it = source.GetEnumerator();
            Assert.IsTrue(it.MoveNext(), "1st word");
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Save and Load that changing the NumberOfSubFiles will create a new files
        /// and delete all old sub-files.
        /// </summary>
        [Test]
        public void TestSaveAndLoadMultipleCleanOldFiles()
        {
            // split into 2 files
            source.FilePath = subFiles;
            source.NumberOfSubFiles = 2;
            source.Save();

            // 2 sub file + 1 main file only is created
            string[] files = Directory.GetFileSystemEntries(@"..\..\test_files", "*EraseMe*.xml");
            Assert.AreEqual(3, files.Length, "3 files are created");

            // split into 1 file only
            source.NumberOfSubFiles = 1;
            source.Save();

            // make sure old files are deleted and we only have 1 sub file + 1 main file now
            files = Directory.GetFileSystemEntries(@"..\..\test_files", "*EraseMe*.xml");
            Assert.AreEqual(2, files.Length, "2 files are created");

            // single file
            source.NumberOfSubFiles = 0;
            source.Save();

            // make sure we only have 1 single file now
            files = Directory.GetFileSystemEntries(@"..\..\test_files", "*EraseMe*.xml");
            Assert.AreEqual(1, files.Length, "1 main file only");

            // reload the files (no exception is expected)
            source.Load(new SimpleIndexParameters(subFiles));

            // 6 unique words
            IDictionaryEnumerator it = source.GetEnumerator();
            for (int i = 0; i < 6; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Entry: " + i);
            }
            Assert.IsFalse(it.MoveNext(), "No more entry");

            // resulting final single file should be like Document0.xml
            string expected = base.ReadContent(docFile, true);
            string actual = base.ReadContent(subFiles, true);
            Assert.AreEqual(expected, actual, "The xml is incorrect");
        }
    }
}

